<?php
/**
  * Novalnet payment method module
  * This module is used for real time processing of
  * Novalnet transaction of customers.
  *
  * Copyright (c) Novalnet AG
  *
  * Released under the GNU General Public License
  * This free contribution made by request.
  * If you have found this script useful a small
  * recommendation as well as a comment on merchant form
  * would be greatly appreciated.
  *
  * @file    : Novalnet Payment Module
  * Script   : uc_novalnet.inc
  *
  */

    module_load_include('inc', 'uc_novalnet', '/includes/uc_novalnet_form');

/**
 * Payment method callback function: payment method information
 * @param  string $nn_payment
 * @return array or string
 */
    function uc_novalnet_payment_methods($nn_payment = '') {
        $basic_data      = basic_params();
        $nn_prefix       =  !empty ( $basic_data['nnlogo']) ? 'Novalnet ' : '';
        $payment_methods = array(
            'novalnet_prepayment' => array(
                    'name' => $nn_prefix . t('Prepayment'),
                    'key'  => '27',
                    'desc' =>  t("Once you've submitted the order, you will receive an e-mail with account details to make payment")),
            'novalnet_invoice'    => array(
                    'name' => $nn_prefix . t('Invoice'),
                    'key'  => '27',
                    'desc' =>  t("Once you've submitted the order, you will receive an e-mail with account details to make payment")),
            'novalnet_cc'         => array(
                    'name' => $nn_prefix . t('Credit Card'),
                    'key'  => '6',
                    'desc' =>  t('The amount will be debited from your credit card once the order is submitted')),
            'novalnet_paypal'     => array(
                    'name' => $nn_prefix . t('PayPal'),
                    'key'  => '34',
                    'desc' =>  t('After the successful verification, you will be redirected to Novalnet secure order page to proceed with the payment')),
            'novalnet_ideal' => array(
                    'name' => $nn_prefix . t('iDEAL'),
                    'key'  => '49',
                    'desc' =>  t('After the successful verification, you will be redirected to Novalnet secure order page to proceed with the payment')),
            'novalnet_eps' => array(
                    'name' => $nn_prefix . t('eps'),
                    'key'  => '50',
                    'desc' =>  t('After the successful verification, you will be redirected to Novalnet secure order page to proceed with the payment')),
            'novalnet_instantbank' => array(
                    'name' => $nn_prefix . t('Instant Bank Transfer'),
                    'key'  => '33',
                    'desc' => t('After the successful verification, you will be redirected to Novalnet secure order page to proceed with the payment')),
            'novalnet_sepa'        => array(
                    'name' => $nn_prefix . t('Direct Debit SEPA'),
                    'key'  => '37',
                    'desc' =>  t('Your account will be debited upon the order submission'))
        );
        return (!empty($nn_payment)) ? $payment_methods[$nn_payment] : $payment_methods;
    }

/**
* Payment method callback function: to get the payport url for relevant payment
* @param  string $payment
* @return string
*/
    function uc_novalnet_get_url($payment) {
        if (variable_get('novalnet_cc_cc3d') && $payment == 'novalnet_cc')
        return 'https://payport.novalnet.de/global_pci_payport';

        switch ($payment) {
            case 'novalnet_paypal':
              return 'https://payport.novalnet.de/paypal_payport';
            case 'novalnet_ideal':
              return 'https://payport.novalnet.de/online_transfer_payport';
            case 'novalnet_eps':
              return 'https://payport.novalnet.de/eps_payport';
            case 'novalnet_instantbank':
              return 'https://payport.novalnet.de/online_transfer_payport';
            case 'nncurl_request':
              return 'https://payport.novalnet.de/paygate.jsp';
            case 'nnxml_request':
              return 'https://payport.novalnet.de/nn_infoport.xml';
        }
    }

/**
* Payment method callback function: to get the title of relevant payment
* @param  string $key
* @param  string $name
* @param  array $basic_data
* @return string
*/
    function uc_novalnet_payment_title($key, $name, $basic_data) {
        $module_url = drupal_get_path('module', 'uc_novalnet');
        $module_url = url('', array('relative' => TRUE)) . $module_url;
        $domain_url = t('http://www.novalnet.com');
        $name = preg_replace('|Novalnet|s', '', $name);
        $nnlogo = (!empty($basic_data['nnlogo'])) ? l(t('<img src="' . $module_url . '/img/novalnet_logo.png" title="Novalnet AG" alt="Novalnet AG" >'), $domain_url, array('html' => TRUE, 'attributes' => array('target' => '_blank'))) : '';
        $payment_logo = variable_get($key . '_paymentlogo', 1) ? l(t('<img src="' . $module_url . '/img/' . $key . '.png" title="' . $name . '" alt="' . $name . '">'), $domain_url, array('html' => TRUE, 'attributes' => array('target' => '_blank'))) :'';
        if ($key == 'novalnet_cc' && variable_get('novalnet_cc_amex')) {
            $payment_logo = (variable_get($key . '_paymentlogo', 1))? '<a href="' . $domain_url . '"><img src="' . $module_url . '/img/novalnet_cc_amex.png" title="' . $name . '" alt="' . $name . '"></a>' :'';
        }
        return $nnlogo . $name . $payment_logo;
    }

/**
* Payment method validate function : to validate the subscription details
* @param  array $element
* @param  array $form_state
* @return none
*/
    function uc_novalnet_config_validation($element, &$form_state) {
        $values = $form_state['values'];
        if ($values['enable_subscription']) {
            if (empty($values['subs_tariffid']))
            form_set_error('subs_tariffid', t('Please enter the valid subscription') . ' ' . ('tariff id'));

            if (empty($values['tariff_period']))
            form_set_error('tariff_period', t('Please enter the valid subscription period'));

            if (($values['tariff_period2_amount']) && empty($values['tariff_period2']))
            form_set_error('tariff_period2', t('Please enter the valid subscription period2'));

            if ($values['tariff_period2'] && !is_numeric($values['tariff_period2_amount']))
            form_set_error('tariff_period2', t('Please enter the valid subscription period2 amount'));
        }
    }

/**
* menu callback : to display the admin portal inside the shop
* @param  none
* @return none
*/
    function uc_novalnet_merchant_admin() {
        $support = t('Login here with Novalnet merchant credentials. For the activation of new payment methods please contact %ssupport@novalnet.de%s');
        echo '<div style="background-color: #0080c9;color: #fff;font-family: calibri;font-size: 16px;font-weight: bold;padding: 5px;text-align: center;">' . sprintf($support, "<a href='mailto:sales@novalnet.de' style='color: #fff'>", "</a>") . '</div><iframe frameborder="0" style="width:100%;height:80%;" src="https://admin.novalnet.de"></iframe>';
        exit;
    }

/**
* Payment method validate function : to validate payment access key
* @param  array $element
* @param  array $form_state
* @return none
*/
    function uc_novalnet_cc3d_validation($element, &$form_state) {
        if ($element['#value'] && empty($form_state['values']['novalnet_cc_key']))
        form_error($element, l(t('Enter the Novalnet payment access key')));
    }
/**
* Reference parameter formation:
* @param  array $element
* @param  array $form_state
* @return none
*/
    function uc_novalnet_referid($element, &$form_state) {
        $form_state['values'] = array_map('strip_tags', $form_state['values']);
        $form_state['values'] = array_map('trim', $form_state['values']);
    }

/**
* Payment method validate function: to validate the numbers
* @param  array $element
* @param  array $form_state
* @return none
*/
    function uc_novalnet_element_validate_number($element, &$form_state)  {
        $element_value = trim($element['#value']);
        if ( $element_value != '') {
            if (!preg_match("/^[0-9]+$/", $element_value)) {
                if (!$element['#title']) $element['#title'] = t('Notice Period');
                $errormsg = $element['#title'] . ' ' . t('is not valid');
                form_error($element, check_plain($errormsg));
            }
        }
    }

/**
* Payment method validate function: to validate the email address
* @param  array $element
* @param  array $form_state
* @return none
*/

    function uc_novalnet_element_validate_email($element, &$form_state) {
        $emails = explode(',', $element['#value']);
        foreach ($emails as $email) {
            if (($form_state['values']['cbk_mail_send'] == 1) && !valid_email_address($email)) {
                $errormsg = $element['#title'] . ' ' . t('is not valid');
                form_error($element, check_plain($errormsg));
            }
        }
    }

/**
* Payment method validate function: to validate the Direct Debit SEPA duedate
* @param  array $element
* @param  array $form_state
* @return none
*/
    function uc_novalnet_element_validate_duedate($element, &$form_state) {
        $payment = $form_state['build_info']['args'][0];
        $element_value = trim($element['#value']);
        if ($payment != 'novalnet_sepa') return TRUE;

        if ($element_value != '') {
            if (!preg_match("/^[0-9]+$/", $element_value) || $element_value <= 6 ) {
                $errormsg = t('SEPA Due date is not valid');
                form_error($element, check_plain($errormsg));
            }
        }
    }
/**
* Payment method callback function: to store the session values for Direct Debit SEPA
* @param  array $element
* @param  array $form_state
* @return none
*/
    function uc_novalnet_nnsepaform_validate($element, &$form_state) {
        $payment = $form_state['values']['panes']['payment']['payment_method'];
        if ($payment != 'novalnet_sepa') return TRUE;

        $config_data = basic_params();
        if ($form_state['values']['panes']['payment']['details']['nn_sepa_hash'] != '' ||  (bool) $config_data['refill']) {
            uc_novalnet_setsession($payment, array(
              'nn_sepa_hash'    => $form_state['values']['panes']['payment']['details']['nn_sepa_hash'],
              'nnsepa_owner'    => $form_state['values']['panes']['payment']['details']['nnsepa_owner'],
              'sepa_from_acno'  => $form_state['values']['panes']['payment']['details']['sepa_from_acno'],
              'nnsepa_uniqueid' => $form_state['values']['panes']['payment']['details']['nn_uniqueid']
            ));
        }
    }

/**
* Payment method validate function: to validate the data of mobile number and email
* @param  array $element
* @param  array $form_state
* @return none
*/
    function uc_novalnet_element_validate_fraud($element, &$form_state) {
        $payment = $form_state['values']['panes']['payment']['payment_method'];
        if (trim($element['#value']) != '') {
            if (($element['#attributes']['id'] == 'fraud_number' && !preg_match("/^[0-9]+$/", trim($element['#value']))) || ($element['#attributes']['id'] == 'fraud_email' && !valid_email_address(trim($element['#value'])))) {
                $errormsg = $element['#title'] . ' ' . t('is not valid');
                form_error($element, check_plain($errormsg));
            }
            uc_novalnet_setsession($payment, array($payment . '_pinkey'  => trim($element['#value'])));
        }
    }

/**
* Payment method callback function: to complete redirection payment
* @param  none
* @return none
*/
    function uc_novalnet_nnpayment_complete() {
        $parsed = $_POST; unset($_POST);
        uc_novalnet_completeorder($parsed);
    }

/**
* Payment method validate function: to validate the pin number
* @param  array $element
* @param  array $form_state
* @return none
*/

    function uc_novalnet_element_validate_pin($element, &$form_state) {
        $payment = $form_state['values']['panes']['payment']['payment_method'];
        $errormsg = t('Pin number is not valid');
        if (!$form_state['values']['panes']['payment']['details'][$payment . '_forget_pin']) {
            if (trim($element['#value']) == '') {
              $errormsg = t('The PIN you entered is incorrect');
              form_error($element, check_plain($errormsg));
            }
            uc_novalnet_setsession($payment, array('pin'  => trim($element['#value']), 'forget_pin' => 0));
        }
        else{
            uc_novalnet_setsession($payment, array('forget_pin'  => trim($form_state['values']['panes']['payment']['details'][$payment . '_forget_pin'])));
        }
    }

/**
* Payment method callback function: to store the session values for Credit Card
* @param  array $element
* @param  array $form_state
* @return mixed
*/
    function uc_novalnet_nnccform_validate($element, &$form_state) {
        $payment = $form_state['values']['panes']['payment']['payment_method'];
        if ($payment != 'novalnet_cc') return TRUE;

        $config_data = basic_params();
        if ($form_state['values']['panes']['payment']['details']['nn_cc_hash'] != '' || (bool) $config_data['refill']) {
            uc_novalnet_setsession($payment, array(
              'nncc_holder'  => $form_state['values']['panes']['payment']['details']['nncc_owner'],
              'cc_type'      => $form_state['values']['panes']['payment']['details']['nncc_type'],
              'pan_hash'     => $form_state['values']['panes']['payment']['details']['nn_cc_hash'],
              'unique_id'    => $form_state['values']['panes']['payment']['details']['nn_uniqueid'],
              'cc_exp_month' => $form_state['values']['panes']['payment']['details']['nncc_exp_month'],
              'cc_exp_year'  => $form_state['values']['panes']['payment']['details']['nncc_exp_year'],
              'cc_cvc2'      => $form_state['values']['panes']['payment']['details']['nncc_cvc']
            ));
        }
    }

/**
* Payment method callback function: to form the defaut parameters
* @param  array $data
* @param  object $order
* @return none
*/
    function uc_novalnet_getparams(&$data, $order) {
        global $language;
        $config_data     = basic_params();
        $billing_country = uc_get_country_data(array('country_id' => $order->billing_country));
        $country         = $billing_country[0]['country_iso_code_2'];
        $ipaddr          = ip_address();
        $data = array(
                    'vendor'           => $config_data['vendor_id'],
                    'product'          => $config_data['product_id'],
                    'tariff'           => $config_data['tariff_id'],
                    'auth_code'        => $config_data['auth_code'],
                    'test_mode'        => variable_get($order->payment_method . '_test_mode', 0),
                    'currency'         => variable_get('uc_currency_code', 'USD'),
                    'first_name'       => trim($order->billing_first_name),
                    'last_name'        => trim($order->billing_last_name),
                    'gender'           => 'u',
                    'email'            => trim($order->primary_email),
                    'street'           => $order->billing_street1,
                    'search_in_street' => 1,
                    'city'             => $order->billing_city,
                    'zip'              => $order->billing_postal_code,
                    'tel'              => trim($order->billing_phone),
                    'lang'             => strtoupper($language->language),
                    'country'          => $country,
                    'country_code'     => $country,
                    'session'          => session_id(),
                    'remote_ip'        => (($ipaddr == '::1') ? '127.0.0.1' : $ipaddr),
                    'order_no'         => $order->order_id,
                    'payment_method'   => $order->payment_method,
                    'customer_no'      => ($order->uid > 0 ) ? $order->uid : 'guest',
                    'use_utf8'         => 1,
                    'birth_date'       => '',
                    'fax'              => '',
                    'system_name'      => 'drupal-übercart',
                    'system_version'   => VERSION . '-3.8-NN10.0.0',
                    'system_ip'        => (($_SERVER['SERVER_ADDR'] == '::1') ? '127.0.0.1' : $_SERVER['SERVER_ADDR']),
                    'system_url'       => url('', array('absolute' => TRUE)),
                    'amount'           => round($order->order_total, 2) * 100
                );
        if (in_array($order->payment_method, array('novalnet_cc', 'novalnet_sepa', 'novalnet_invoice')))
            uc_novalnet_checklimit($data, $order->payment_method, $order->order_total);

        if ((bool) $config_data['enable_subscription'] && in_array($order->payment_method, $config_data['subscription_payments']))
            uc_novalnet_recurring_data($data, $order->payment_method);

        uc_novalnet_additional_params($data, $order->payment_method, $config_data['payment_access_key']);
        if (!empty($order->billing_street2))
            $data['street'] .= ' ' . $order->billing_street2;
        if (!empty($config_data['referrer_id']))
            $data['referer_id'] = $config_data['referrer_id'];
    }

/**
* Payment method callback function: to form the subscription parameters
* @param  array $data
* @param  array $payment
* @return array
*/
    function uc_novalnet_recurring_data(&$data, $payment) {
        $basic_data                    = basic_params();
        $data['tariff']                = $basic_data['subs_tariffid'];
        $data['tariff_period']         = $basic_data['tariff_period'];
        $data['tariff_period2']        = $basic_data['tariff_period2'];
        $data['tariff_period2_amount'] = $basic_data['tariff_period2_amount'];
        return str_replace(array('days', 'months', 'years'), array('d', 'm', 'y'), $data);
    }
/**
* Payment method callback function: to complete Novalnet order
* @param  array $parsed
* @param  array $order
* @return none
*/
    function uc_novalnet_completeorder($parsed = array(), $order = array()) {
        if (empty($parsed) && !empty($_POST))
        $parsed =  $_POST;
        $status_message = uc_novalnet_check_status($parsed);
        if (empty($order)) $order = uc_order_load($parsed['order_no']);
        $parsed['payment_method'] = $order -> payment_method;
        if ($status_message != 'success') {
            drupal_set_message(check_plain($status_message), 'error');
            watchdog('uc_novalnet', '<br> Payment Error: @error', array('@error' => $status_message), WATCHDOG_ERROR);
            drupal_goto('cart/checkout');
        }
        if (in_array($parsed['payment_method'], array('novalnet_paypal', 'novalnet_ideal', 'novalnet_instantbank', 'novalnet_eps'))) {
            $config_data = basic_params();
            uc_novalnet_checkHash($parsed, $config_data['payment_access_key']);
            uc_novalnet_decode_param($parsed, $config_data['payment_access_key']);
        }
        $parsed['nn_comments'] = uc_novalnet_get_comments($parsed);
        if (isset($parsed['product_id'])) $parsed['product'] = $parsed['product_id'];
        uc_novalnet_ordercomplete($parsed, $order);
        drupal_set_message($parsed['nn_comments']);
        uc_novalnet_postback($parsed);
        uc_novalnet_update_status($parsed);
        $_SESSION['uc_checkout'][$_SESSION['cart_order']]['do_complete'] = TRUE;
        $_SESSION['do_complete'] = TRUE;
        drupal_goto('cart/checkout/complete');
    }

/**
* Payment method callback function: for onhold product
* @param  array $data
* @param  array $payment
* @param int $order_total
* @return none
*/
    function uc_novalnet_checklimit(&$data, $payment, $order_total) {
        $limit = basic_params();
        if ($limit['manualamount'] > 0 && $limit['manualamount'] <= ($order_total * 100)) $data['on_hold'] = 1;
    }

/**
* Payment method callback function:  to form the additional parameters
* @param  array $data
* @param  array $payment_method
* @param int $key
* @return none
*/
    function uc_novalnet_additional_params(&$data, $payment_method, $key) {
        global $user, $base_url;
        $payment = uc_novalnet_payment_methods($payment_method);
        $data['key'] = $payment['key'];
        if (variable_get($payment_method . '_ref1', '') != '' ) {
            $data['input1']     = 'reference1';
            $data['inputval1']  = variable_get($payment_method . '_ref1');
        }
        if (variable_get($payment_method . '_ref2', '') != '' ) {
            $data['input2']     = 'reference2';
            $data['inputval2']  = variable_get($payment_method . '_ref2');
        }
        switch ($payment_method) {
            case 'novalnet_invoice':
                $data['invoice_ref']    = "BNR-" . $data['product'] . "-" . $data['order_no'];
                $data['invoice_type']   = 'INVOICE';
                $due_date = (variable_get($payment_method . '_due_date')=='')? 14 : variable_get($payment_method . '_due_date');
                $data['due_date']     = date('Y-m-d', strtotime("+ $due_date day"));
              break;
            case 'novalnet_prepayment':
                $data['invoice_ref']    = "BNR-" . $data['product'] . "-" . $data['order_no'];
                $data['invoice_type']         = 'PREPAYMENT';
              break;
            case 'novalnet_cc':
                $data['cc_holder']            = $_SESSION['novalnet_session']['novalnet_cc']['nncc_holder'];
                $data['cc_no']                = '';
                $data['cc_type']              = $_SESSION['novalnet_session']['novalnet_cc']['cc_type'];
                $data['pan_hash']             = $_SESSION['novalnet_session']['novalnet_cc']['pan_hash'];
                $data['unique_id']            = $_SESSION['novalnet_session']['novalnet_cc']['unique_id'];
                $data['cc_exp_month']         = $_SESSION['novalnet_session']['novalnet_cc']['cc_exp_month'];
                $data['cc_exp_year']          = $_SESSION['novalnet_session']['novalnet_cc']['cc_exp_year'];
                $data['cc_cvc2']              = $_SESSION['novalnet_session']['novalnet_cc']['cc_cvc2'];
                if (variable_get('novalnet_cc_cc3d')) {
                    $data['encoded_amount'] = $data['amount'];
                    uc_novalnet_encode_data($data, $payment_method, $key);
                }
              break;
            case 'novalnet_ideal':
            case 'novalnet_eps':
            case 'novalnet_instantbank':
            case 'novalnet_paypal':
                $data['uniqid'] = uniqid();
                uc_novalnet_encode_data($data, $payment_method, $key);
                $data['hash'] = uc_novalnet_create_hash($data, $key);
                $data['user_variable'] = $base_url;

              break;
            case 'novalnet_sepa':
                $due_date = (variable_get($payment_method . '_due_date') == '')? 14 : variable_get($payment_method . '_due_date');
                $data['sepa_due_date']        = date('Y-m-d', strtotime("+ $due_date day"));
                $data['sepa_unique_id']       = $_SESSION['novalnet_session']['novalnet_sepa']['nnsepa_uniqueid'];
                $data['sepa_hash']            = $_SESSION['novalnet_session']['novalnet_sepa']['nn_sepa_hash'];
                $data['bank_account_holder']  = $_SESSION['novalnet_session']['novalnet_sepa']['nnsepa_owner'];
                $data['bank_account']         = '';
                $data['bank_code']            = '';
                $data['bic']                  = '';
                $data['iban']                 = '';
                if ((bool) variable_get('sepa_refill', 0)) variable_set('sepa_refill' . $user->uid, $data['sepa_hash']);
              break;

        }
    }

/**
* Payment method callback function: set encoded params
* @param  array $data
* @param  array $payment_method
* @param int $key
* @return none
*/
    function uc_novalnet_encode_data(&$data, $payment_method, $key) {
        $encode_data = array();
        if (in_array($payment_method, array('novalnet_paypal', 'novalnet_ideal', 'novalnet_instantbank', 'novalnet_eps')))
            $encode_data = array('product',  'tariff',  'test_mode',  'uniqid',  'amount', 'auth_code');
        if ($payment_method == 'novalnet_cc')
            $encode_data = array('encoded_amount');
        foreach ($encode_data as $encode_element) {
            $data[$encode_element] = uc_novalnet_encoded_value($data[$encode_element], $key);
        }
    }

/**
* Payment method callback function: encode the payment params
* @param $element
* @param int $key
* @return none
*/
    function uc_novalnet_encoded_value($element, $key) {
        try {
            $crc = sprintf('%u',  crc32($element));
            return strrev( base64_encode( bin2hex( $crc . "|" . $element . $key ) ) );
        }
        catch (Exception $e) {
            drupal_set_message(check_plain($e), 'error');
            drupal_goto('cart/checkout');
            return FALSE;
        }
    }

/**
* Payment method callback function: set decode params
* @param array $data
* @param int $key
* @return none
*/
    function uc_novalnet_decode_param(&$data, $key) {
        $encode_data = array('product',  'tariff',  'test_mode',  'uniqid',  'amount', 'auth_code');
        foreach ($encode_data as $value) {
            $data[$value] = uc_novalnet_decode_values($data[$value], $key);
        }
    }

/**
* Payment method callback function: decode the payment param
* @param array $element
* @param int $key
* @return none
*/
    function uc_novalnet_decode_values($element, $key) {
        try{
            $data =  base64_decode(strrev($element));
            $data = pack("H" . strlen($data),  $data);
            $data = substr($data,  0,  stripos($data,  $key));
            $pos  = strpos($data,  "|");
            if ($pos === FALSE) $result = FALSE;

            $crc   = substr($data,  0,  $pos);
            $value = trim(substr($data,  $pos+1));
            if ($crc != sprintf('%u',  crc32($value))) $result = FALSE;
            if (isset($result) && !$result) {
                drupal_set_message(t('Decode Error'), 'error');
                drupal_goto('cart/checkout');
            }
            return $value;
        }
        catch (Exception $e) {
            drupal_set_message(check_plain($e), 'error');
            drupal_goto('cart/checkout');
        }
    }

/**
* Payment method callback function: hash generation
* @param array $data
* @param int $key
* @return array
*/
    function uc_novalnet_create_hash($data, $key) {
        $hash_string = '';
        $hash_string .= $data['auth_code'];
        $hash_string .= $data['product'];
        $hash_string .= $data['tariff'];
        $hash_string .= $data['amount'];
        $hash_string .= $data['test_mode'];
        $hash_string .= $data['uniqid'];
        return md5($hash_string . strrev($key));
    }

/**
* Payment method callback function: to check the payment hash
* @param array $parsed
* @param int $key
* @return bool
*/
    function uc_novalnet_checkHash($parsed, $key) {
        if ($parsed['hash2'] != uc_novalnet_create_hash($parsed, $key)) {
            drupal_set_message(t('While redirecting some data has been changed. The hash check failed.'), 'error');
            drupal_goto('cart/checkout');
        }
        return TRUE;
    }

/**
* Payment method callback function: to check the payment status
* @param array $response
* @return string
*/
    function uc_novalnet_check_status($response) {
        if (isset($response['status'])) {
            if ($response['status'] == 100 || ($response['status'] == 90 &&  $response['payment_method'] == 'novalnet_paypal')) return 'success';
            elseif (isset($response['status_text'])) return $response['status_text'];
            elseif (isset($response['status_desc'])) return  $response['status_desc'];
            elseif (isset($response['novaltel_status_message'])) return  $response['novaltel_status_message'];
            elseif (isset($response['status_message'])) {
                if ($response['status'] == '0529006')
                $_SESSION['novalnet_error_payment'] = array($response['payment_method'] => array('error_time' => time()));
                return  $response['status_message'];
            }
        }
        elseif (isset($response['pin_status'])) return $response['pin_status']['status_message'];
    }

/**
* Payment method callback function: to append the payent comments
* @param array $data
* @return array $comment
*/
    function uc_novalnet_get_comments($data, $new_order='') {
        $test_mode = ($data['test_mode'] == 1) ? t('Test order') :'';
        $payment   = uc_novalnet_payment_methods($data['payment_method']);
        $comment   = $test_mode . '<br />' . $payment['name'] . '<br />' . t('Novalnet transaction ID :') . ' ' . $data['tid'] . '<br />';
        if (!empty($new_order)) {
        $basic_data = basic_params();
        $data['order_no']       = $new_order->order_id;
        $data['invoice_ref']    =  "BNR-" . $basic_data['product_id'] . "-" . $data['order_no'];
        }
        if (in_array($data['payment_method'], array('novalnet_invoice', 'novalnet_prepayment')) || (!empty($data['invoice_det']))) {
            $nn_prepayment_comments = ($data['payment_method'] == 'novalnet_prepayment') ? '<div id=nn_prepayment_comments>' : '';
            $comments_end = ($data['payment_method'] == 'novalnet_prepayment') ? '</div>' : '';
            $comment .= $nn_prepayment_comments . t('Please transfer the amount to the below mentioned account details of our payment processor Novalnet') . '<br />';
            $comment .= t('Due date : @due_date', array('@due_date' => $data['due_date'])) . '<br/>' .
                        t('Account holder : @comp', array('@comp' => 'NOVALNET AG')) . '<br/>' .
                        t('IBAN : @invoice_iban', array('@invoice_iban' => $data['invoice_iban'])) . '<br/>' .
                        t('BIC : @invoice_bic', array('@invoice_bic' => $data['invoice_bic'])) . '<br/>' .
                        t('Bank : @bank', array('@bank' => $data['invoice_bankname'] . ' ' . $data['invoice_bankplace'])) . '<br/><span id="amount_update">' .
                        t('Amount : @amount', array('@amount' => uc_currency_format($data['amount']/100))) . '</span><br />' .
                        t('Reference 1 : @ref1', array('@ref1' => $data['invoice_ref'])) . '<br/>' .
                        t('Reference 2 : @ref2 @tid', array('@ref2' => t('TID '), '@tid' => $data['tid'])) . '<br/>' .
                        t('Reference 3 : @ref3 @order_id', array('@ref3' => t('Order number'), '@order_id' => $data['order_no'])) . $comments_end;
        }
        return $comment;
    }

/**
* Payment method callback function: to set order status
* @param array $data
* @param object $order
* @return bool
*/
    function uc_novalnet_ordercomplete($data, $order) {
        if (in_array($data['payment_method'], array('novalnet_invoice', 'novalnet_prepayment')) || ($data['status'] == 90 && $data['payment_method'] == 'novalnet_paypal')) {
            $order->order_total = 0;
            $order_status = variable_get($data['payment_method'] . '_before_status', 'pending');
        }
        else $order_status = variable_get($data['payment_method'] . '_after_status', 'completed');

        uc_order_comment_save($order->order_id,  0, $data['nn_comments'] ,  'order', $order_status , 'TRUE');
        uc_payment_enter($order->order_id,  $data['payment_method'],  $order->order_total,  $order->uid,  NULL,  $data['nn_comments']);
        uc_order_update_status($order->order_id, $order_status);
        return TRUE;
    }

/**
* Payment method callback function: to send the acknowledgement to the payment server
* @param array $data
* @return none
*/
    function uc_novalnet_postback($data) {
        if (isset($data['nn_comments'])) unset($data['nn_comments']);
        uc_novalnet_save_log($data['order_no'], $data, 'response');

        global $user;
        $request = array(
            'vendor'    => $data['vendor'],
            'product'   => $data['product'],
            'tariff'    => $data['tariff'],
            'auth_code' => $data['auth_code'],
            'order_no'  => $data['order_no'],
            'key'       => $data['key'],
            'status'    => 100,
            'tid'       => $data['tid']
        );

        if ($data['key'] == 27)
        $request['invoice_ref'] =  "BNR-" . $data['product'] . "-" . $data['order_no'];
        uc_novalnet_send_payport($request, 'curl');
        uc_novalnet_setsession();
        $_SESSION['nn_payment'] = $data['payment_method'];
    }

/**
* Payment method callback function: to update the transaction status
* @param array $data
* @param string $callback
* @return none
*/
    function uc_novalnet_update_status($data, $callback = '') {
        $basic_data = basic_params();
        $request = array(
            'vendor_id'       => $basic_data['vendor_id'],
            'vendor_authcode' => $basic_data['auth_code'],
            'request_type'    => 'TRANSACTION_STATUS',
            'product_id'      => $basic_data['product_id'],
            'tid'             => !empty($callback) ? $callback : $data['tid']
        );
        $response = uc_novalnet_send_payport($request);
        if (!empty($callback)) return $response;

        $update_data = array('tid' => $data['tid'], 'status' => $response['status'], 'key' => $data['key'], 'amount' => $response['amount']);
        $basic_array = array('vendor_id', 'auth_code', 'product_id', 'tariff_id');
        foreach ($basic_array as $key) {
            $update_data[$key] = $basic_data[$key];
        }
        if ($data['key'] == 27)
        $update_data['due_date'] = $data['due_date'];
        if ($basic_data['enable_subscription'] && in_array($data['payment_method'], $basic_data['subscription_payments'])) {
            $update_data['subscription'] = $data['tid'];
            $cancel_data = array('order_id' => $data['order_no'], 'tid' => $data['tid'], 'ref_orders' => $data['order_no']);
            update_variable(array(), $cancel_data, 'uc_novalnet_cancel');
        }
        update_variable(array('order_id' => $data['order_no']), $update_data, 'uc_novalnet_transaction');
        if (!empty($_SESSION['nn']['nn_aff_id'])) {
            if (!empty($_SESSION['nn']['update'])) {
                global $user;
                update_variable(array('aff_id' => $_SESSION['nn']['nn_aff_id']), array('aff_id' => $_SESSION['nn']['nn_aff_id'], 'customer_id' => $user->uid, 'aff_order_no' => $data['order_no']), 'nn_aff_user_detail');
            }
            unset($_SESSION['nn']);
        }
    }

/**
* Payment method callback function: to save the transaction details made from the shop
* @param array $data
* @param string $order_id
* @param string $type
* @return none
*/
    function uc_novalnet_save_log($order_id, $data, $type) {
        if ($type == 'request' && isset($data['key'])) {
            if ($data['key'] == 6) {
                $data['pan_hash'] = '****';
                $data['cc_exp_month'] = '****';
                $data['cc_exp_year'] =  '****';
                $data['cc_cvc2'] = '****';
            }
            elseif ($data['key'] == 37) {
                $data['sepa_hash'] = '****' ;
            }
        }

        $db_data = array(
            'id' => '',
            'order_id' => $order_id,
            'logtype'  => $type,
            'logvalue' => serialize($data),
            'logtime'  => time()
        );
        drupal_write_record('uc_novalnet_logs', $db_data);
    }

/**
* Payment method callback function: request to the payment server
* @param array $request
* @param string $type (is curl or xml)
* @return mixed
*/
    function uc_novalnet_send_payport($request, $type = '') {
        if (array_search('', array_map('trim', $request)) && $type != 'curl') {
            drupal_set_message(t('Required param missing!'), 'error');
            return FALSE;
        }
        if (isset($request['order_no']) && empty($request['tid'])) {
            uc_novalnet_save_log($request['order_no'], $request, 'request');
        }
        if ($type == 'curl') {
            $xml  = http_build_query($request, '', '&');
            $host = uc_novalnet_get_url('nncurl_request');
        }
        else {
            $xml = "<?xml version='1.0' encoding='UTF-8'?>
                <nnxml>
                <info_request>";
            foreach ($request as $tag => $val) {
                $xml .= "<" . $tag . ">" . $val . "</" . $tag . ">";
            }
            $xml .= "</info_request>
                </nnxml>";
            $host = uc_novalnet_get_url('nnxml_request');
        }
        $options = array(
            'headers'   => array('Content-Type' => 'application/x-www-form-urlencoded'),
            'method'    => 'POST',
            'data'      => $xml,
            'timeout'   => 90.0
        );
        $response = drupal_http_request($host, $options);
        if ($type == 'curl')
        return isset($response->data) ? $response->data : 'status=408&status_desc=Novalnet Payment server is busy Please try after some time';
        else
        return isset($response->data) ? json_decode(json_encode(simplexml_load_string($response->data)), TRUE) : 'status=408&status_desc=Novalnet Payment server is busy Please try after some time';
    }

/**
* Payment method callback function: unique-id generation function
* @param string $charLimit
* @return string
*/
    function randomstring($charlimit = '30') {
        $randomwordarray=explode(",", "a,b,c,d,e,f,g,h,i,j,k,l,m,1,2,3,4,5,6,7,8,9,0");
        shuffle($randomwordarray);
        return substr(implode($randomwordarray, ""), 0, $charlimit);
    }
/**
* Payment method callback function: to maintain payment information in session, while using fraud prevention
* @param object $order
* @return string
*/
    function uc_novalnet_storervalues( $order ) {
        $payment      = $order->payment_method;
        $fraud_method = variable_get($payment . '_pin_method', '');

        if ($fraud_method && isset($_SESSION['novalnet_session'][$payment][$payment . '_pinkey'])) {
            $data = array();
            uc_novalnet_getparams($data, $order);
            if (!isset($_SESSION['novalnet_session'][$payment]['tid'])) {
                if (variable_get($payment . '_pin_method')) {
                    $fraud_array = array( 'callback'  => array('pin_by_callback', 'tel'),
                                          'sms'       => array('pin_by_sms', 'mobile'),
                                          'email'     => array('reply_email_check', 'email'));
                    $data[$fraud_array[$fraud_method][0]] = 1;
                    $data[$fraud_array[$fraud_method][1]] = $_SESSION['novalnet_session'][$payment][$payment . '_pinkey'];
                }
                if (variable_get($payment . '_pin_amount', '') && sprintf('%0.2f', $order->order_total) * 100 < variable_get($payment . '_pin_amount')) {
                    drupal_set_message(t('The order amount has been changed, please proceed with the new order'), 'error');
                    drupal_goto('cart/checkout');
                }
                $response = uc_novalnet_send_payport($data, 'curl');
                parse_str($response, $parsed);
                $status_message = uc_novalnet_check_status($parsed);
                if ($status_message != 'success') {
                    drupal_set_message(check_plain($status_message), 'error');
                    watchdog('uc_novalnet', '<br> Payment Error: @error', array('@error' => $status_message), WATCHDOG_ERROR);
                    uc_novalnet_setsession($payment);
                }
                else {
                    $session_array = array('tid' => $parsed['tid'], 'amount' => $parsed['amount'] * 100, 'test_mode' => $parsed['test_mode']);
                    if ($payment == 'novalnet_invoice')
                    $session_array = array_merge($session_array, array('invoice_bankcode' => $parsed['invoice_bankcode'],
                                                                   'invoice_iban'       => $parsed['invoice_iban'],
                                                                   'due_date'           => $parsed['due_date'],
                                                                   'invoice_bic'        => $parsed['invoice_bic'],
                                                                   'invoice_bankname'   => $parsed['invoice_bankname'],
                                                                   'invoice_bankplace'  => $parsed['invoice_bankplace']));
                    $message_array = array('callback'  => t('You will shortly receive a transaction PIN through phone call to complete the payment'),
                                        'sms'  => t('You will shortly receive an SMS containing your transaction PIN to complete the payment'),
                                        'email'  => t('You will shortly receive an information e-mail, please send the empty reply incl. the original e-mail'),
                    );
                    $message = $message_array[$fraud_method];
                    uc_novalnet_setsession($payment, $session_array);
                    drupal_set_message(check_plain($message), 'warning');
                }
                drupal_goto('cart/checkout');
            }
            else{
                if (isset($_SESSION['novalnet_session'][$payment]['forget_pin']) && $_SESSION['novalnet_session'][$payment]['forget_pin']) {
                    $xml_request = array('vendor_id' => $data['vendor'],
                                  'vendor_authcode' => $data['auth_code'],
                                  'tid'             => $_SESSION['novalnet_session'][$payment]['tid'],
                                  'lang'            => $data['lang'],
                                  'request_type'    => 'TRANSMIT_PIN_AGAIN'
                                   );
                    $parsed = uc_novalnet_send_payport($xml_request);
                    drupal_set_message($parsed['status_message'], 'warning');
                    drupal_goto('cart/checkout');
                }
            }
        }
        if (!isset($_SESSION['novalnet_session'][$payment])) uc_novalnet_setsession();
        else uc_novalnet_setsession($payment, $_SESSION['novalnet_session'][$payment], TRUE);
    }

/**
* Payment method callback function: to store the session details and also to unset them.
* @param string $payment
* @param array $values
* @param bool $reset
* @return none
*/
    function uc_novalnet_setsession($payment = '' , $values = array(), $reset = FALSE) {
        if ((isset($_SESSION['novalnet_session']) && empty($payment)) || $reset) unset($_SESSION['novalnet_session']);
        if (!empty($values)) {
            $_SESSION['novalnet_session'][$payment] = isset($_SESSION['novalnet_session'][$payment]) ? array_merge( $_SESSION['novalnet_session'][$payment], $values) : $values;
        }
    }

/**
* Payment method callback function: to cancel the subscription.
* @param  none
* @return none
*/
    function uc_novalnet_subscription_cancel() {
        $basic_data = basic_params(); $data = $_REQUEST;
        $result     = uc_novalnet_send_payport($data, $type = 'curl');
        if ($data || isset($result['status'])) {
            $order = get_orginal_details($data['tid']);
            subscription_cancel_update($order->order_id, t('Subscription has been canceled due to:') . ' ' . $data['cancel_reason'], $data['cancel_reason']);
            echo json_encode(array('hash_result' => 'success'));
        }
        else echo json_encode(array('hash_result' => 'restricted area'));
    }

/**
* Payment method callback function: to update the callback amount
* @param object $order
* @param string $messages
* @param string $due_date
* @return string
*/
    function callback_amt_update($order, $messages, $due_date='') {
        $updated_data = nn_variable_get(array('order_id' => $order->order_id), 'uc_novalnet_transaction');  $value = '';
        $result = db_select('uc_payment_receipts', 'r')
                 ->fields('r', array('comment'))
                 ->condition('order_id', $order->order_id)
                 ->execute()
                 ->fetchAssoc();
        if (in_array($order->payment_method, array('novalnet_invoice', 'novalnet_prepayment'))) {
        $result['comment'] = preg_replace("/\d{2}\/\d{2}\/\d{4}/", !empty($due_date) ? $due_date : $updated_data['due_date'] , $result['comment']);
        if (!empty($_SESSION['novalnet_session'][$order->payment_method]['updated_amount']))
        $result['comment'] = preg_replace('|<span id="amount_update">(.+?)</span>|s', t('Amount : @amount', array('@amount' => uc_currency_format($_SESSION['novalnet_session'][$order->payment_method]['updated_amount']))), $result['comment']);
        }
        $value = $result['comment'] . "<br/>";
        return $value . $messages;
    }

/**
* Payment method callback function: to update the columns in payment table
* @param int $id
* @param array $data
* @param string $table
* @return none
*/
    function update_variable($id, $data, $table) {
        if (count($data) < 3)
            db_update($table)->fields($data)->condition(key($id), current($id))->execute();

        else {
            if ($table == 'uc_novalnet_transaction') {
                $data['order_id']   = current($id);
                $data['paymentkey'] = $data['key']; unset($data['key']);
            }
            db_insert($table)->fields($data)->execute();
        }
    }

/**
* Payment method callback function: to select the columns in payment table
* @param array $id
* @param string $table
* @return array
*/
    function nn_variable_get($id, $table) {
        return db_select($table, 'nn')
                ->fields('nn')
                ->condition(key($id), current($id))
                ->execute()
                ->fetchAssoc();
    }

/**
* Payment method callback function: update the subscription status
* @param string $orderid
* @param string $comment
* @param string $reason
* @return none
*/
    function subscription_cancel_update($orderid, $comment, $reason) {
        $basic_data = variable_get('novalnet_params', '');
        $result = db_select('uc_novalnet_cancel', 'arg')
            ->fields('arg', array('ref_orders'))
            ->condition('order_id', $orderid)
            ->execute()
            ->fetchcol();
        foreach ($result as $order_id) {
            $order = uc_order_load($order_id);
            uc_order_comment_save($order->order_id,  $order->uid,  $comment , 'order', $basic_data['subscription_cancel_status'], 'TRUE');
            update_variable(array('order_id' => $order->order_id), array('subscription' => 0), 'uc_novalnet_transaction');
            update_variable(array('order_id' => $order->order_id), array('reason_id' => $reason), 'uc_novalnet_cancel');
        }
        uc_order_update_status($orderid, $basic_data['subscription_cancel_status']);
    }

/**
* Payment method callback function: to fetch the transaction details
* @param string $tid
* @return object
*/
    function get_orginal_details($tid) {
        return db_select('uc_novalnet_transaction', 'arg')
                -> fields('arg', array('order_id'))
                -> condition('tid', $tid)
                -> execute()
                -> fetchObject();
    }

/**
* implementation of hook_init()
* @params none
* @return none
*/
    function uc_novalnet_init() {
        if (drupal_is_front_page()) {
            $request = $_REQUEST;
            if (!empty($request['nn_aff_id'])) {
                $_SESSION['nn']['nn_aff_id']  = $request['nn_aff_id'];
                $_SESSION['nn']['update']     = TRUE;
            }
            else {
                global $user;
                $aff_details = nn_variable_get(array('customer_id' => $user->uid), 'nn_aff_user_detail');
                if (!empty($aff_details['aff_id'])) $_SESSION['nn']['nn_aff_id'] = $aff_details['aff_id'];
            }
        }
    }

/**
* Payment method callback function : to fetch the basic params
* @params none
* @return array
*/
    function basic_params() {
        $basic_params = variable_get('novalnet_params', '');
        if (!empty($_SESSION['nn']['nn_aff_id'])) {
            $aff_details = nn_variable_get(array('aff_id' => $_SESSION['nn']['nn_aff_id']), 'nn_aff_account_detail');
            $basic_params['vendor_id']          = $aff_details['aff_id'];
            $basic_params['auth_code']          = $aff_details['aff_authcode'];
            $basic_params['payment_access_key'] = $aff_details['aff_accesskey'];
        }
        return $basic_params;
    }
