<?php
/**
  * Novalnet payment method module
  * This module is used for real time processing of
  * Novalnet transaction of customers.
  *
  * Copyright (c) Novalnet AG
  *
  * Released under the GNU General Public License
  * This free contribution made by request.
  * If you have found this script useful a small
  * recommendation as well as a comment on merchant form
  * would be greatly appreciated.
  *
  * @file    : Novalnet Payment Module
  * Script   : uc_novalnet_admin.inc
  *
  */

    module_load_include('inc', 'uc_novalnet', '/includes/uc_novalnet_form');

/**
* implementation of hook_menu()
* @params none
* @return array $items
*/
    function uc_novalnet_menu() {
        $items = array();
        $items['novalnet_order/review'] = array(
            'title'            => 'Review payment',
            'page callback'    => 'uc_novalnet_nnpayment_complete',
            'access arguments' => array('access content'),
            'type'             => MENU_CALLBACK,
            'file'             => 'includes/uc_novalnet.inc',
        );
        $items['callback_novalnet2ubercart'] = array(
            'title'            => 'callbackscript',
            'page callback'    => 'uc_novalnet_callbackscript',
            'access arguments' => array('access content'),
            'type'             => MENU_CALLBACK,
            'file'             => 'includes/uc_novalnet_page.inc',
        );
        $items['admin/novalnet'] = array(
            'title'            => 'Novalnet',
            'menu_name'        => 'management',
            'description'      => 'Novalnet' . ' ' . t('Configuration'),
            'page callback'    => 'system_admin_menu_block_page',
            'access arguments' => array('administer site configuration'),
            'file'             => 'system.admin.inc',
            'file path'        => drupal_get_path('module', 'system'),
        );
        $items['admin/novalnet/globalconfig'] = array(
            'title callback'   => 'global_config_title',
            'title arguments'  => array('Novalnet Global Configuration'),
            'weight'           => 2,
            'description'      => '',
            'page callback'    => 'drupal_get_form',
            'page arguments'   => array('uc_novalnet_global_config_form'),
            'access callback'  => TRUE,
            'file'             => 'includes/uc_novalnet_form.inc',
        );
        $items['admin/novalnet/novalnet_orders'] = array(
            'title'            => 'Novalnet' . ' ' . t('Orders'),
            'page callback'    => 'drupal_goto',
            'access arguments' => array('Extension'),
            'page arguments'   => array('admin/store/novalnet_orders/view'),
            'weight'           => -99
        );
        $items['admin/novalnet/novalnet_orders/%uc_order/novalnet_log'] = array(
            'title'            => 'Novalnet Transaction Log',
            'page callback'    => 'uc_novalnet_payment_logs',
            'page arguments'   => array(3),
            'access arguments' => array('view all orders'),
            'type'             => MENU_LOCAL_TASK,
            'weight'           => 10,
            'file'             => 'includes/uc_novalnet_form.inc',
        );
        $items['merchant_paypal_config'] = array(
            'title'            => '',
            'type'             => MENU_NORMAL_ITEM,
            'page callback'    => 'drupal_get_form',
            'page arguments'   => array('uc_novalnet_merchant_admin'),
            'access callback'  => TRUE,
        );
        $items['novalnet_order/subs_cancel'] = array(
            'title'            => 'subscription cancel',
            'page callback'    => 'uc_novalnet_subscription_cancel',
            'access callback'  => TRUE,
            'access arguments' => array('access content'),
            'type'             => MENU_CALLBACK,
            'file'             => 'includes/uc_novalnet.inc',
        );
        return $items;
    }

/**
* implementation of title callback()
* @params none
* @return string
 */
    function global_config_title($txt) {
      return t($txt);
    }
/**
* implementation of hook_views_api()
* @params none
* @return array
 */
    function uc_novalnet_views_api() {
        return array(
            'api'  => 3,
            'path' => drupal_get_path('module', 'uc_novalnet') . '/views',
        );
    }

/**
* implementation of hook_form_alter()
* @params array $form
* @params array $form_state
* @params string $form_id
* @return none
 */
    function uc_novalnet_form_alter(&$form, &$form_state, $form_id) {
        global $user;
        if ($form_id == 'uc_payment_methods_form') {
            $config_data = basic_params();
            $uc_payments = $form['pmtable'];
            $nn_payments = uc_novalnet_payment_methods();
            $error = FALSE;
            if (empty($config_data)) {
                foreach ($nn_payments as $key => $val) {
                    $form['pmtable'][$key]['#disabled'] = TRUE;
                }
                $error = 'basic';
            }
            elseif (empty($config_data['payment_access_key'])) {
                $pass_methods    = array('novalnet_paypal', 'novalnet_ideal', 'novalnet_instantbank', 'novalnet_eps');
                foreach ($pass_methods as $val) {
                    $form['pmtable'][$val]['#disabled'] = TRUE;
                }
                $error = 'password';
            }
            if ($error) drupal_set_message(t('Please configure') . ' ' . l(t('Novalnet Global Configuration'), 'admin/novalnet/globalconfig') . ' ' . t('to enable the Novalnet Payment methods') , 'warning');
        }
        if ($form_id == 'uc_order_view_update_form') {
            $form['#attributes']['autocomplete'] = 'off';
            $order        = uc_order_load($form['order_id']['#value']);
            $basic_config = basic_params();
            $tid_details  = nn_variable_get(array('order_id' => $order->order_id), 'uc_novalnet_transaction');

            $voidcapture        = array('novalnet_invoice', 'novalnet_prepayment', 'novalnet_cc', 'novalnet_sepa');
            $amount_update    = array('novalnet_invoice', 'novalnet_prepayment');
            $allowed_payments = array('novalnet_cc', 'novalnet_sepa', 'novalnet_paypal', 'novalnet_ideal', 'novalnet_instantbank', 'novalnet_eps');

            if (in_array($tid_details['status'], array(91, 99, 98, 100))) {

                if (in_array($tid_details['status'], array(91, 99, 98)) && in_array($order->payment_method, $voidcapture)) {
                    $form['nn_void_capture'] = array(
                        '#type'        => 'fieldset',
                        '#title'       => t('Manage Transaction'),
                        '#collapsible' => 1,
                        '#collapsed'   => 1,
                        '#attributes'  => array('id' => 'void_capture_fieldset')
                    );
                    $form['nn_void_capture']['on_hold_transaction_enable'] = array(
                       '#type'        => 'checkbox',
                       '#title'       => t('Manage transaction process'),
                       '#attributes'  => array('id' => 'transaction_enable', 'onclick' => 'enable_mode("amount_update_fieldset");')
                    );
                    $form['nn_void_capture']['on_hold_transaction'] = array(
                        '#type'          => 'select',
                        '#options'       => array('' => t('--Select--'), 'void' => t('Cancel'), 'capture' => t('Confirm')),
                        '#default_value' => t('--select--'),
                        '#states'        => array('visible' => array('input[id="transaction_enable"]' => array('checked'      => TRUE))),
                    );
                }
                if (((in_array($order->payment_method, $amount_update) && $tid_details['status'] == '100') || ($order->payment_method == 'novalnet_sepa' && $tid_details['status'] == '99')) && (int) $tid_details['amount'] > 0) {
                    $title = ($order->payment_method == 'novalnet_sepa') ?t('Amount update'):t('Change the amount / due date ');
                    $form['nn_amount_update'] = array(
                        '#type'        => 'fieldset',
                        '#title'       => check_plain($title),
                        '#collapsible' => 1,
                        '#collapsed'   => 1,
                        '#attributes'  => array('id' => 'amount_update_fieldset')
                    );
                    $form['nn_amount_update']['nn_amount_update_enable']  = array(
                        '#type'        => 'checkbox',
                        '#title'       => check_plain($title),
                        '#attributes'  => array('id' => 'enable_amount_update', 'onclick' => 'enable_mode("void_capture_fieldset");')
                    );
                    $form['nn_amount_update']['transaction_amount_change']  = array(
                        '#type'        => 'textfield',
                        '#title'       => t('Update transaction amount') . ' ' . t('in cents') ,
                        '#attributes'  => array('onkeypress' => 'return ccnumber_validate(event)'),
                        '#description' => check_plain(t('Novalnet transaction ID :') . $tid_details['tid']),
                        '#states'      => array('visible' => array('input[id="enable_amount_update"]' => array('checked'      => TRUE))),
                        '#default_value' => $tid_details['amount']
                    );

                    if (in_array($order->payment_method, $amount_update))
                    $form['nn_amount_update']['invoice_duedate_change']  = array(
                        '#type'   => 'textfield',
                        '#title'  => t('Transaction due date'),
                        '#default_value' => isset($tid_details['due_date']) ? $tid_details['due_date'] : date('d.m.Y'),
                        '#states' => array('visible' => array('input[id="enable_amount_update"]' => array('checked' => TRUE))),
                    );
                }

                if ($tid_details['status'] == 100 && in_array($order->payment_method, $allowed_payments) || !empty($tid_details['callback_amount'])) {
                    $form['nn_refund'] = array(
                        '#type'        => 'fieldset',
                        '#title'       => t('Manage Transaction'),
                        '#collapsible' => 1,
                        '#collapsed'   => 1,
                    );
                    $form['nn_refund']['nn_refund_enable']  = array(
                        '#type'        => 'checkbox',
                        '#title'       => t('Transaction refund'),
                        '#attributes'  => array('id' => 'enable_nn_refund' , 'onclick' => 'enable_mode("amount_update_fieldset");')
                    );
                    $form['nn_refund']['transaction_refund']  = array(
                       '#type'          => 'textfield',
                       '#title'         => t('Please enter the refund amount') . ' ' . t('(in cents)'),
                       '#description'   => check_plain('Novalnet ' . t('Transaction ID :') . $tid_details['tid']),
                       '#attributes'   => array('onkeypress' => 'return ccnumber_validate(event)'),
                       '#default_value' => !empty($tid_details['callback_amount']) ? $tid_details['callback_amount'] :  $tid_details['amount'],
                       '#states'        => array('visible' => array('input[id="enable_nn_refund"]' => array('checked'      => TRUE))),
                    );
                    if (in_array($order->payment_method, array('novalnet_invoice', 'novalnet_prepayment', 'novalnet_ideal', 'novalnet_instantbank'))) {
                        $form['nn_refund']['enable_ddsepa_refund'] = array(
                            '#type'   => 'checkbox',
                            '#title'  => t('Direct Debit SEPA'),
                            '#states' => array('visible' => array('input[id="enable_nn_refund"]' => array('checked' => TRUE))),
                        );
                        $form['nn_refund']['account_holder']  = array(
                           '#type'   => 'textfield',
                           '#title'  => t('Account holder :'),
                           '#states' => array('visible' => array('input[name="enable_ddsepa_refund"]' => array('checked' => TRUE))),
                           '#default_value' => $order->billing_first_name . ' ' . $order->billing_last_name,
                        );
                        $form['nn_refund']['sepa_iban']  = array(
                           '#type'   => 'textfield',
                           '#title'  => t('IBAN:'),
                           '#states' => array('visible' => array('input[name="enable_ddsepa_refund"]' => array('checked' => TRUE))),
                        );
                        $form['nn_refund']['sepa_bic']  = array(
                           '#type'   => 'textfield',
                           '#title'  => t('BIC'),
                           '#states' => array('visible' => array('input[name="enable_ddsepa_refund"]' => array('checked' => TRUE))),
                        );
                    }
                        $visible =  (time() >= strtotime("+1 days", $order->created)) ? TRUE : FALSE ;
                        if ($visible) {
                        $form['nn_refund']['refund_ref']  = array(
                           '#type'   => 'textfield',
                           '#title'  => t('Refund reference'),
                        );
                        }
                }
                $form['#submit'][] = 'uc_refund_process_submit';
            }
        }
    }

/**
* Payment method callback function: submit handler
* @params array $form
* @params array $form_state
* @return none
 */
    function uc_refund_process_submit($form, &$form_state) {
        $form_values  = array_map('trim', $form_state['values']); $type = '';
        $order        = uc_order_load($form['order_id']['#value']);
        $basic_config = basic_params();
        $comments = '';
        $tid_details  = nn_variable_get(array('order_id' => $order->order_id), 'uc_novalnet_transaction');
        $transaction_call = array(
            'vendor'          => $tid_details['vendor_id'],
            'auth_code'       => $tid_details['auth_code'],
            'product'         => $tid_details['product_id'],
            'tariff'          => $tid_details['tariff_id'],
            'key'             => $tid_details['paymentkey'],
            'tid'             => $tid_details['tid'],
            'status'          => $tid_details['status'],
        );

        if (!empty($form_values['on_hold_transaction_enable'])) {
        if ($form_values['on_hold_transaction'] == '')
        return FALSE;
            $type = 'onhold_transaction_status';
            $transaction_call['edit_status'] = 1;
            $transaction_call['status']      = 100;
            if ($form_values['on_hold_transaction'] == 'void') {
                $transaction_call['status'] = 103;
                $type = 'onhold_void_cancel_status';
            }
            $order_status    = $basic_config[$type];
            $update_variable = array('status' => $transaction_call['status']);
            $comments        = ($type == 'onhold_void_cancel_status') ? t('The transaction has been canceled on @date, @time', array('@date' => date('Y:m:d'), '@time' => date('h:i:s'))) : t('Transaction confirmed successfully');
        }

        elseif (!empty($form_values['nn_amount_update_enable'])) {
            if (!is_numeric($form_values['transaction_amount_change']) || (!empty($tid_details['callback_amount']) && ($form_values['transaction_amount_change'] < $tid_details['callback_amount']))) {
                form_set_error('transaction_amount_change', t('Amount should be greater than paid amount' . ' ' . uc_currency_format($tid_details['callback_amount']/100)));
                drupal_goto('admin/store/orders/' . $order->order_id);
            }

            $type = 'amount_update';
            $transaction_call['status']                = 100;
            $transaction_call['amount']                = $form_values['transaction_amount_change'];
            $transaction_call['update_inv_amount']     = 1;
            $transaction_call['edit_status']           = 1;
            $transaction_call['due_date']              = ($tid_details['paymentkey'] == 27 && isset($form_values['invoice_duedate_change'])) ? date('Y-m-d', strtotime($form_values['invoice_duedate_change'])) : '0000-00-00';
            $amt = $form_values['transaction_amount_change'];
            if (in_array($order->payment_method, array('novalnet_invoice', 'novalnet_prepayment')) && isset($tid_details['callback_amount'])) {
            $amt = $form_values['transaction_amount_change'] - $tid_details['callback_amount'];
            $amount_to_update = ($tid_details['callback_amount'] == 0 && $tid_details['amount'] <= $tid_details['callback_total']) ?  0: $amt;
            }
            $update_variable = array('amount' => $amt, 'due_date' => date('m/d/Y', strtotime($transaction_call['due_date'])));
            $comments        = t('The Transaction amount @amount has been updated successfully on @date,@time', array('@amount' => uc_currency_format($update_variable['amount'] /100), '@date' => date('Y:m:d'), '@time' => date('h:i:s')));
        }
        elseif (!empty($form_values['nn_refund_enable'])) {
            if ((!empty($form_values['enable_ddsepa_refund']) && empty($form_values['sepa_iban']) && empty($form_values['sepa_bic'])) || empty($form_values['transaction_refund'])) {
            if (empty($form_values['transaction_refund']))
            form_set_error('transaction_refund', t('Please enter the refund amount'));
            form_set_error('transaction_refund', t('Your account details are invalid'));
            drupal_goto('admin/store/orders/' . $order->order_id);
            }
            $amount = !empty($tid_details['callback_amount']) ? $tid_details['callback_amount'] : $tid_details['amount'];
            if ($form_values['transaction_refund'] > $amount) {
                form_set_error('transaction_refund', t('Refund cannot be processed, as the amount exceeds the paid amount!'));
                drupal_goto('admin/store/orders/' . $order->order_id);
            }
            $type = 'refund';
            $transaction_call['status'] = 100;
            $transaction_call['refund_request'] = 1;
            $transaction_call['refund_param']   = $form_values['transaction_refund'];
            if (!empty($form_values['refund_ref']))
            $transaction_call['refund_ref']   = $form_values['refund_ref'];
            if (isset($form_values['enable_ddsepa_refund']) && $form_values['enable_ddsepa_refund']) {
                $transaction_call['account_holder'] = $form_values['account_holder'];
                $transaction_call['account_no']     = $form_values['sepa_iban'];
                $transaction_call['bank_code']      = $form_values['sepa_bic'];
            }
            if (in_array($order->payment_method, array('novalnet_invoice', 'novalnet_prepayment'))) {
                $amt = $tid_details['callback_amount'] - $form_values['transaction_refund'];
                $cancel_status = (empty($amt) && empty($tid_details['amount'])) ? TRUE : FALSE;
                $update_variable = array('callback_amount' => $amt);
                $update_amt = TRUE;
            }
            else {
                $amt = $tid_details['amount'] - $form_values['transaction_refund'];
                $update_variable = array('amount' => $amt);
                $cancel_status = empty($amt) ? TRUE : FALSE;
            }
            if ($cancel_status) {
                $order_status = $basic_config['onhold_void_cancel_status'];
                $update_variable = (in_array($order->payment_method, array('novalnet_invoice', 'novalnet_prepayment'))) ? array('status' => 103, 'callback_amount' => 0 ) : array('status' => 103, 'amount' => 0 );
            }
            $comments = sprintf(t('The refund has been executed for the TID: %s with the amount %s'), $transaction_call['tid'], uc_currency_format($transaction_call['refund_param']/100));
        }
        if (isset($update_variable)) {
            $response = uc_novalnet_send_payport($transaction_call, 'curl');
            parse_str($response, $parsed);
            if ($parsed['status'] == 100) {
                if (!empty($update_amt)) {
                $up_amount = $tid_details['amount'] - $form_values['transaction_refund'];
                update_variable(array('order_id' => $order->order_id), array('amount' => abs($up_amount)), 'uc_novalnet_transaction');
                }
                if (($type == 'refund') && !empty($parsed['tid']))
                $comments = sprintf(t('The refund has been executed for the TID: %s with the amount of %s Your New TID:%s for the refunded amount %s'), $transaction_call['tid'], uc_currency_format($transaction_call['refund_param']/100), $parsed['tid'], uc_currency_format($transaction_call['refund_param']/100));
                $order_status = !empty($order_status) ? $order_status : $order->order_status;
                if (!empty($transaction_call['amount']))
                $_SESSION['novalnet_session'][$order->payment_method]['updated_amount'] = $transaction_call['amount']/100;
                $comments     = callback_amt_update($order, $comments, ($type == 'amount_update')? $update_variable['due_date'] : '');
                uc_order_comment_save($order->order_id,  0,  $comments ,  'order',  $order_status, 'TRUE');
                uc_order_update_status($order->order_id, $order_status);
                update_variable(array('order_id' => $order->order_id), $update_variable, 'uc_novalnet_transaction');
                if (isset($amount_to_update) && $amount_to_update===0)
                update_variable(array('order_id' => $order->order_id), array('amount' => $amount_to_update), 'uc_novalnet_transaction');
                if (isset($_SESSION['novalnet_session'][$order->payment_method]['updated_amount']))
                unset($_SESSION['novalnet_session'][$order->payment_method]['updated_amount']);
                drupal_set_message($comments);
            }
            else isset($parsed['status_desc']) ? drupal_set_message($parsed['status_desc'], 'error') : '';
        }
    }

/**
* implementation of hook_uc_order_pane()
* @params none
* @return array
 */
    function uc_novalnet_uc_order_pane() {
        $panes['subscription'] = array(
            'callback' => 'novalnet_subs_cancel',
            'title'    => t('Cancel Subscription Process'),
            'desc'     => t("Manage the order's shipping address and contact information."),
            'class'    => 'pos-left',
            'id'       => 'hide',
            'weight'   => -7,
            'show'     => array('view', 'edit', 'customer'),
        );
        return $panes;
    }

/**
* Payment method callback function: subscription cancel
* @params string $op
* @params object $order
* @params array $form
* @params array $form_state
* @return array $form
 */
    function novalnet_subs_cancel($op, $order, &$form = NULL, &$form_state = NULL) {
        global $base_url;
        $basic_data   = basic_params();
        $tid_details  = nn_variable_get(array('order_id' => $order->order_id), 'uc_novalnet_transaction');
        $module_url   = '/' . drupal_get_path('module', 'uc_novalnet');
        $request = array(
            'vendor_id'       => $basic_data['vendor_id'],
            'vendor_authcode' => $basic_data['auth_code'],
            'request_type'    => 'TRANSACTION_STATUS',
            'product_id'      => $basic_data['product_id'],
            'tid'             => $tid_details['tid']
        );
        $response = uc_novalnet_send_payport($request);
        if (($tid_details['subscription']) && ( isset($response['status']) && $response['status'] != 103)) {
            $options  = array(t('Please select reason'), t('Product is costly'), t('Cheating'), t('Partner interfered'), t('Financial problem'), t('Content does not match my likes'), t('Content is not enough'), t('Interested only for a trial'), t('Page is very slow'), t('Satisfied customer'), t('Logging in problems'), t('Other'));
            switch ($op) {
            case 'customer':
            case 'view':
                global $base_url;
                $form['cancel_status'] = array(
                    '#type'           => 'select',
                    '#options'        => $options,
                    '#title'          => t('Please select the reason of subscription cancellation :'),
                    '#attributes'     => array('id' => 'uc_novalnet_subs_cancel_reason'),
                );
                $form['base_url'] = array(
                    '#type'           => 'hidden',
                    '#attributes'     => array('id' => 'base_url'),
                    '#value'          => $base_url . '/novalnet_order/subs_cancel'
                );
                $form['cancel_msg'] = array(
                    '#type'           => 'hidden',
                    '#attributes'     => array('id' => 'cancel_msg'),
                    '#value'          => t('Subscription has been canceled due to:')
                );
                $form['vendor_details_cancel'] = array(
                    '#type'           => 'hidden',
                    '#attributes'     => array('id' => 'subs_cancel_vendor_details'),
                    '#value'          => $tid_details['vendor_id'] . '&' . $tid_details['auth_code'] . '&' . $tid_details['product_id'] . '&' . $tid_details['tariff_id'] . '&' . $tid_details['subscription'] . '&' . $tid_details['paymentkey'] . '&' . t('Please select the reason of subscription cancellation :') . '&' . $base_url . '/' . current_path()
                );
                $form['cancel_submit'] = array(
                    '#type'           => 'button',
                    '#value'          => t('Cancel Subscription'),
                    '#prefix'         => '<div id ="loading-img"><img src="' . $module_url . '/img/novalnet_loader.gif"></div>',
                    '#attributes'     => array('id' => 'uc_novalnet_subs_cancel'),
                );
                return $form;
            }
        }
    }

/**
* Implementation of hook_mail
* @params string $key
* @params array $params
* @params array $message
* @return none
 */
    function uc_novalnet_mail($key, &$message, $params) {
        $message['headers']['Content-Type'] = "text/html; charset=UTF-8; format=flowed; delsp=yes";
        $message['headers'] += $params['headers'];
        $message['subject']  = $params['subject'];
        $message['body'][]   = $params['body'];
    }
/**
* Implementation of hook_mail_alter
* @params array $message
* @return none
 */
    function uc_novalnet_mail_alter(&$message) {
        $message['headers']['Content-Type'] = 'text/html; charset=UTF-8; format=flowed';
        if ($message['id'] == 'uc_order_invoice' && $message['params']['order']->payment_method == 'novalnet_prepayment')
        $message['body'][0] = preg_replace('|<div id=nn_prepayment_comments>(.+?)</div>|s', ' ', $message['body'][0]);
    }

/**
* Implementation of hook_cron()
* @params none
* @return none
*/
    function uc_novalnet_cron() {
        $cron_last = variable_get('cron_last');
        if ( date('d', $cron_last) != date('d', time() ) ) {
            drupal_flush_all_caches();
        }
    }
