<?php
/**
  * Novalnet payment method module
  * This module is used for real time processing of
  * Novalnet transaction of customers.
  *
  * Copyright (c) Novalnet AG
  *
  * Released under the GNU General Public License
  * This free contribution made by request.
  * If you have found this script useful a small
  * recommendation as well as a comment on merchant form
  * would be greatly appreciated.
  *
  * @file    : Novalnet Payment Module
  * Script   : uc_novalnet_form.inc
  *
  */

    module_load_include('inc', 'uc_novalnet', 'includes/uc_novalnet_admin');

/**
* Implementation of global configuration for novalnet payments
* @param  array $form
* @param  array $form_state
* @return array $form
*/   

    function uc_novalnet_global_config_form($form, &$form_state) {
        $base_url = url('', array('relative' => TRUE));
        $admin_portal  = t("For additional configurations login to %sNovalnet Administration Portal.%s To login to the Portal you need to have an account at Novalnet. If you don't have one yet, please contact %ssales@novalnet.de%s / tel. +49 (089) 923068320");
        $paypal_config = t('To use the PayPal payment method please enter your PayPal API details in %sNovalnet Merchant Administration%s portal');
        $form['merchant_details'] = array(
            '#markup' => sprintf($admin_portal, "<a href='" . $base_url . "merchant_paypal_config'>", "</a>", "<a href='mailto:sales@novalnet.de'>", "</a>") . '<br/>'
        );
        $form['paypal_account'] = array(
            '#markup' => sprintf($paypal_config, "<a href='" . $base_url . "merchant_paypal_config'>", "</a>") . '<br/>'
        );
        $form['novalnet_settings'] = array( 
            '#type' => 'vertical_tabs',
        );
        $basic_data = basic_params();
        $cbk_data   = variable_get('novalnet_cbk_data', '');
        $payments   =  array(
            'novalnet_cc'           => t('Credit Card'),
            'novalnet_sepa'         => t('Direct Debit SEPA'),
            'novalnet_prepayment'   => t('Prepayment'),
            'novalnet_invoice'      => t('Invoice'),
        );
        $form['global_config'] = array(
            '#type'        => 'fieldset',
            '#title'       => t('Global Configuration'),
            '#collapsible' => TRUE,
            '#collapsed'   => TRUE,
            '#group'       => 'novalnet_settings',
        );
        $form['callback_config'] = array(
            '#type'        => 'fieldset',
            '#title'       => t('Merchant script management'),
            '#collapsible' => TRUE,
            '#collapsed'   => TRUE,
            '#group'       => 'novalnet_settings',
        );
        $form['global_config']['nnlogo'] = array(
            '#type'          => 'select',
            '#title'         => t('Display Novalnet logo'),
            '#description'   => t('The Novalnet logo will be displayed on the checkout page'),
            '#options'       => array(1 => t('True'), 0 => t('False')),
            '#default_value' => isset($basic_data['nnlogo'])? $basic_data['nnlogo'] : 1
        );
        $form['global_config']['vendor_id'] = array(
            '#type'             => 'textfield',
            '#description'      => t('Enter Novalnet merchant ID'),
            '#title'            => t('Merchant ID'),
            '#default_value'    => isset($basic_data['vendor_id'])? $basic_data['vendor_id'] : '',
            '#required'         => TRUE,
            '#element_validate' => array('uc_novalnet_element_validate_number'),
        );
        $form['global_config']['auth_code'] = array(
            '#type'             => 'textfield',
            '#description'      => t('Enter Novalnet authentication code'),
            '#title'            => t('Authentication code'),
            '#default_value'    => isset($basic_data['auth_code'])? $basic_data['auth_code'] : '',
            '#required'         => TRUE,
        );
        $form['global_config']['product_id'] = array(
            '#type'             => 'textfield',
            '#description'      => t('Enter Novalnet project ID'),
            '#title'            => t('Project ID'),
            '#default_value'    => isset($basic_data['product_id'])? $basic_data['product_id'] : '',
            '#required'         => TRUE,
            '#element_validate' => array('uc_novalnet_element_validate_number'),
        );
        $form['global_config']['tariff_id'] = array(
            '#type'             => 'textfield',
            '#description'      => t('Enter Novalnet tariff ID'),
            '#title'            => t('Tariff ID'),
            '#required'         => TRUE,
            '#default_value'    => isset($basic_data['tariff_id'])? $basic_data['tariff_id'] : '',
            '#element_validate' => array('uc_novalnet_element_validate_number'),
        );
        $form['global_config']['payment_access_key'] = array(
            '#type'                 => 'textfield',
            '#description'          => t('Enter the Novalnet payment access key'),
            '#title'                => t('Payment access key'),
            '#default_value'        => isset($basic_data['payment_access_key'])? $basic_data['payment_access_key'] : '',
            '#required'             => TRUE,
        );
        $form['global_config']['refill'] = array(
            '#type'             => 'select',
            '#title'            => t('Enable auto-fill'),
            '#description'      => t('The payment details will be filled automatically in the payment form during the checkout process'),
            '#default_value'    => isset($basic_data['refill'])? $basic_data['refill'] : 0,
            '#options'          => array( 1 => t('True'), 0 => t('False')),
            '#required'         => FALSE,
        );
        $form['global_config']['manualamount'] = array(
            '#type'          => 'textfield',
            '#title'         => t('Set a limit for on-hold transaction (in cents)'),
            '#description'   => t('In case the order amount exceeds mentioned limit, the transaction will be set on hold till your confirmation of transaction'),
            '#default_value' => (isset($basic_data['manualamount']))? $basic_data['manualamount'] : '',
            '#required'      => FALSE,
            '#element_validate' => array('uc_novalnet_element_validate_number'),
        );
        $form['global_config']['onhold_transaction_status'] = array(
            '#type'          => 'select',
            '#title'         => t('Confirmation order status'),
            '#options'       => uc_order_status_options_list(),
            '#default_value' => (isset($basic_data['onhold_transaction_status']))? $basic_data['onhold_transaction_status'] : 'pending',
        );
        $form['global_config']['onhold_void_cancel_status'] = array(
            '#type'          => 'select',
            '#title'         => t('Cancellation order status'),
            '#options'       => uc_order_status_options_list(),
            '#default_value' => (isset($basic_data['onhold_void_cancel_status']))? $basic_data['onhold_void_cancel_status'] : 'canceled',
        );
        $form['global_config']['referrer_id'] = array(
            '#type'             => 'textfield',
            '#title'            => t('Referrer ID'),
            '#description'      => t('Enter the referrer ID of the person/company who recommended you Novalnet'),
            '#default_value'    => isset($basic_data['referrer_id'])? $basic_data['referrer_id'] : '',
            '#element_validate' => array('uc_novalnet_element_validate_number'),
            '#required'         => FALSE,
        );
        $form['subs_config'] = array(
            '#type'             => 'fieldset',
            '#title'            => t('Dynamic subscription management'),
            '#collapsible'      => TRUE,
            '#collapsed'        => TRUE,
            '#group'            => 'novalnet_settings',
        );
        $form['subs_config']['enable_subscription'] = array(
            '#type'             => 'checkbox',
            '#title'            => t('Configure the Susbcription values'),
            '#default_value'    => isset($basic_data['enable_subscription'])? $basic_data['enable_subscription'] : '',
            '#element_validate' => array('uc_novalnet_config_validation'),
        );
        $form['subs_config']['subscription_payments'] = array(
            '#type'          => 'select',
            '#title'         => t('Subscription payments'),
            '#options'       => $payments,
            '#multiple'      => TRUE,
            '#default_value' => (isset($basic_data['subscription_payments']))? $basic_data['subscription_payments'] : '',
            '#states'        => array('visible' => array('input[id="edit-enable-subscription"]' => array('checked' => TRUE)))
        );
        $form['subs_config']['subscription_info'] = array(
            '#type'          => 'textfield',
            '#title'         => t('Subscription Information'),
            '#description'   => t('Please note: This information will be displayed on other than the Subscription payments.'),
            '#default_value' => (isset($basic_data['subscription_info']))? $basic_data['subscription_info'] : t('This payment will not process the order as Subscription'),
            '#states'        => array('visible' => array('input[id="edit-enable-subscription"]' => array('checked' => TRUE)))
        );
        $form['subs_config']['subs_tariffid'] = array(
            '#type'             => 'textfield',
            '#title'            => t('Novalnet subscription tariff ID'),
            '#description'      => t('Enter your Novalnet subscription tariff ID'),
            '#default_value'    => (isset($basic_data['subs_tariffid'])) ? $basic_data['subs_tariffid'] : '',
            '#element_validate' => array('uc_novalnet_element_validate_number'),
            '#states'           => array('visible' => array('input[id="edit-enable-subscription"]' => array('checked' => TRUE)))
        );
        $form['subs_config']['tariff_period'] = array(
            '#type'          => 'textfield',
            '#title'         => t('Tariff period'),
            '#description'   => t('The period of the first subscription cycle (E.g: 1d/1m/1y)'),
            '#default_value' => (isset($basic_data['tariff_period']))? $basic_data['tariff_period'] : '',
            '#required'      => FALSE,
            '#states'        => array('visible' => array('input[id="edit-enable-subscription"]' => array('checked' => TRUE))),
        );
        $form['subs_config']['tariff_period2_amount'] = array(
            '#type'          => 'textfield',
            '#title'         => t('Amount for the subsequent subscription cycle (in cents)'),
            '#description'   => t('The amount for the subsequent subscription cycle'),
            '#default_value' => (isset($basic_data['tariff_period2_amount']))? $basic_data['tariff_period2_amount'] : '',
            '#states'        => array('visible' => array('input[id="edit-enable-subscription"]' => array('checked' => TRUE))),
        );
        $form['subs_config']['tariff_period2'] = array(
            '#type'          => 'textfield',
            '#title'         => t('Period for subsequent subscription cycle'),
            '#description'   => t('The period of the subsequent subscription cycle (E.g: 1d/1m/1y)'),
            '#default_value' => (isset($basic_data['tariff_period2']))? $basic_data['tariff_period2'] : '',
            '#states'        => array('visible' => array('input[id="edit-enable-subscription"]' => array('checked' => TRUE))),
        );
        $form['subs_config']['subscription_cancel_status'] = array(
            '#type'          => 'select',
            '#title'         => t('Cancellation status of subscription'),
            '#options'       => uc_order_status_options_list(),
            '#default_value' => (isset($basic_data['subscription_cancel_status']))? $basic_data['subscription_cancel_status'] : 'canceled',
            '#states'           => array('visible' => array('input[id="edit-enable-subscription"]' => array('checked' => TRUE)))
        );
        $form['callback_config']['cbk_mail_send'] = array(
            '#type'              => 'select',
            '#title'             => t('Enable E-mail notification for callback'),
            '#options'           => array(0 => t('False'), 1 => t('True')),
            '#default_value'     => (!empty($cbk_data))? $cbk_data['cbk_mail_send'] : 0,
            '#description'       => t('Enable your E-mail send option'),
            '#attributes'        => array('id' => 'nn_cbk_mail_send'),
        );
        $form['callback_config']['cbk_mail_test_mode'] = array(
            '#type'          => 'select',
            '#title'         => t('Enable test mode'),
            '#options'       => array(1 => t('True'), 0 => t('False')),
            '#default_value'    => (!empty($cbk_data['cbk_mail_test_mode']))? $cbk_data['cbk_mail_test_mode'] : 0,
            '#states'           => array('visible' => array('select[id="nn_cbk_mail_send"]' => array('value' => 1)))
        );
        $form['callback_config']['cbk_mail_debug'] = array(
            '#type'          => 'select',
            '#title'         => t('Enable debug mode'),
            '#options'       => array(1 => t('True'), 0 => t('False')),
            '#description'   => t('Set the debug mode to execute the merchant script in debug mode'),
            '#default_value'     => (!empty($cbk_data['cbk_mail_debug']))? $cbk_data['cbk_mail_debug'] : 0,
            '#states'            => array('visible' => array('select[id="nn_cbk_mail_send"]' => array('value' => 1)))
        );
        $form['callback_config']['cbk_mail_from'] = array(
            '#type'              => 'textfield',
            '#title'             => t('E-mail from address'),
            '#default_value'     => (!empty($cbk_data) && !empty($cbk_data['cbk_mail_from']))? $cbk_data['cbk_mail_from'] : variable_get('site_mail'),
            '#element_validate'  => array('uc_novalnet_element_validate_email'),
            '#states'            => array('visible' => array('select[id="nn_cbk_mail_send"]' => array('value' => 1)))
        );
        $form['callback_config']['cbk_mail_to'] = array(
            '#type'              => 'textfield',
            '#title'             => t('E-mail address (To)'),
            '#description'       => t('E-mail address of the recipient'), 
            '#default_value'     => (!empty($cbk_data))? $cbk_data['cbk_mail_to'] : '',
            '#element_validate'  => array('uc_novalnet_element_validate_email'),
            '#states'            => array('visible' => array('select[id="nn_cbk_mail_send"]' => array('value' => 1)))
        );
        $form['callback_config']['cbk_mail_bcc'] = array(
            '#type'           => 'textfield',
            '#required'       => FALSE,
            '#title'          => t("E-mail address (Bcc)"),
            '#description'    => t('E-mail address of the recipient for BCC'), 
            '#default_value'  => (!empty($cbk_data['cbk_mail_bcc']))? $cbk_data['cbk_mail_bcc'] : '',
            '#states'         => array('visible' => array('select[id="nn_cbk_mail_send"]' => array('value' => 1)))
        );
        $form['callback_config']['cbk_mail_subject'] = array(
            '#type'              => 'textfield',
            '#title'             => t('E-mail subject'),
            '#default_value'     => !empty($cbk_data['cbk_mail_subject']) ? $cbk_data['cbk_mail_subject'] : t('Novalnet Callback Script Access Report'),
            '#states'            => array('visible' => array('select[id="nn_cbk_mail_send"]' => array('value' => 1)))
        );
        $form['callback_config']['cbk_mail_body'] = array(
            '#type'              => 'textarea',
            '#title'             => t('E-mail body'),
            '#default_value'     => (!empty($cbk_data))? $cbk_data['cbk_mail_body'] : '',
            '#states'            => array('visible' => array('select[id="nn_cbk_mail_send"]' => array('value' => 1)))
        );
        $form['submit']  = array(
            '#type'             => 'submit',
            '#value'            => t('Save configuration'),
            '#submit'           => array('uc_novalnet_globalconfig_form_submit'),
        );
        return $form;
    }
    
/**
* Submit handler for Global configuration form 
* @param  array $form
* @param  array $form_state
* @return none
*/ 

    function uc_novalnet_globalconfig_form_submit($form, &$form_state) {
        $subs_payments = $form_state['values']['subscription_payments']; unset($form_state['values']['subscription_payments']);
        $config =  array_map('trim' , $form_state['values']);
        variable_set('novalnet_params', array(
            'nnlogo'                     => $config['nnlogo'],
            'vendor_id'                  => $config['vendor_id'],
            'auth_code'                  => $config['auth_code'],
            'product_id'                 => $config['product_id'],
            'tariff_id'                  => $config['tariff_id'],
            'payment_access_key'         => $config['payment_access_key'],
            'referrer_id'                => $config['referrer_id'],
            'enable_subscription'        => $config['enable_subscription'],
            'subscription_payments'      => $subs_payments,
            'subscription_info'          => $config['subscription_info'],
            'subs_tariffid'              => $config['subs_tariffid'],
            'tariff_period'              => $config['tariff_period'],
            'tariff_period2_amount'      => $config['tariff_period2_amount'],
            'tariff_period2'             => $config['tariff_period2'],
            'manualamount'               => $config['manualamount'],
            'refill'                     => $config['refill'],
            'onhold_transaction_status'  => $config['onhold_transaction_status'],
            'onhold_void_cancel_status'  => $config['onhold_void_cancel_status'],
            'subscription_cancel_status' => $config['subscription_cancel_status'],
        ));
        variable_set('novalnet_cc_key', $config['payment_access_key']);
        variable_set('novalnet_cbk_data', array(
            'cbk_mail_send'          => $config['cbk_mail_send'],
            'cbk_mail_from'          => $config['cbk_mail_from'],
            'cbk_mail_to'            => $config['cbk_mail_to'],
            'cbk_mail_subject'       => $config['cbk_mail_subject'],
            'cbk_mail_bcc'           => $config['cbk_mail_bcc'],
            'cbk_mail_debug'         => $config['cbk_mail_debug'],
            'cbk_mail_test_mode'     => $config['cbk_mail_test_mode'],
            'cbk_mail_body'          => $config['cbk_mail_body'],
        ));
        drupal_set_message(t('Your Basic Configuration has been saved.') . ' ' . t('Please configure') . ' ' . t('Payment methods') . ' ' . l(t('Here'), 'admin/store/settings/payment'));
    }

/**
* Back end configuration for Individual payments
* @param  array $payment
* @return array $form
*/   

    function uc_novalnet_backend_config($payment) {
        $form_methods    = array('novalnet_sepa', 'novalnet_cc');
        $pass_methods    = array('novalnet_paypal', 'novalnet_ideal', 'novalnet_instantbank', 'novalnet_eps');
        $duedate_methods = array('novalnet_invoice', 'novalnet_sepa');
        $pin_method      = array('novalnet_sepa', 'novalnet_cc', 'novalnet_invoice');

        $config_data = basic_params();
        if (empty($config_data)) {
            drupal_set_message(t('Please configure Novalnet Global Configuration'), 'error');
            drupal_goto('admin/novalnet/globalconfig');
        }
        $form = array();
        $form['nn_title'] = array(
            '#title' => t('Novalnet Payment Settings'),
        );
        $form[$payment . '_test_mode'] = array(
            '#type'                 => 'select',
            '#title'                => t('Enable test mode'),
            '#description'          => t('The payment will be processed in the test mode therefore amount for this transaction will not be charged'),
            '#default_value'        => variable_get($payment . '_test_mode', 0),
            '#options'              => array( 1 => t('True'), 0 => t('False')),
        );
        $form[$payment . '_paymentlogo'] = array(
            '#type'           => 'select',
            '#title'          => t('Display payment method logo'),
            '#description'    => t('The payment method logo will be displayed on the checkout page'),
            '#default_value'  => variable_get($payment . '_paymentlogo', 1),
            '#options'        => array( 1 => t('True'), 0 => t('False')),
        );
        if ($payment == 'novalnet_cc') {
        $form['novalnet_cc_cc3d'] = array(
            '#type'          => 'checkbox',
            '#title'         => '<b>' . t('Enable 3D secure') . '</b>',
            '#description'   => t('The 3D-Secure will be activated for credit cards. The issuing bank prompts the buyer for a password what, in turn, help to prevent a fraudulent payment. It can be used by the issuing bank as evidence that the buyer is indeed their card holder. This is intended to help decrease a risk of charge-back.'),
            '#default_value' => variable_get('novalnet_cc_cc3d', ''),
            '#element_validate' => array('uc_novalnet_cc3d_validation'),
        );
        $form['novalnet_cc_key'] = array(
            '#type'           => 'hidden',
            '#default_value'  => variable_get('novalnet_cc_key', '')
        );
        $form['novalnet_cc_amex'] = array(
            '#type'                 => 'select',
            '#title'         => t('Enable AMEX'),
            '#default_value' => variable_get('novalnet_cc_amex', 0),
            '#options'              => array( 1 => t('True'), 0 => t('False')),
        );
        $form['novalnet_cc_year_limit'] = array(
            '#type'          => 'textfield',
            '#title'         => t('Limit for expiry year'),
            '#description'   => t('Enter the number for the maximum limit of credit card expiry year. In case if the field is empty, limit of 25 years from the current year will be set by default'),
            '#default_value' => variable_get('novalnet_cc_year_limit', 25),
        );
        }
        if ($payment == 'novalnet_sepa')
        $form['global_config']['sepa_refill'] = array(
            '#type'             => 'select',
            '#title'            => t('Enable auto-fill for payment data'),
            '#description'      => t('For the registered users SEPA direct debit details will be filled automatically in the payment form'),
            '#options'          => array( 1 => t('True'), 0 => t('False')),
            '#default_value'    => variable_get('sepa_refill', 0),
        );

        $cat_option = uc_order_status_options_list();
        if (in_array($payment, array('novalnet_prepayment', 'novalnet_invoice', 'novalnet_paypal'))) { 
        $form[$payment . '_before_status'] = array(
            '#type'          => 'select',
            '#title'         => ($payment == 'novalnet_paypal') ? t('Order status for the pending payment') : t('Order completion status'),
            '#options'       => $cat_option,
            '#default_value' => variable_get($payment . '_before_status', 'pending'),
        );
        }
        $title = (in_array($payment, array('novalnet_prepayment', 'novalnet_invoice'))) ? t('Callback order status') : t('Order completion status') ;
        $form[$payment . '_after_status'] = array(
            '#type'          => 'select',
            '#title'         => $title,
            '#options'       => $cat_option,
            '#default_value' => variable_get($payment . '_after_status', 'completed'),
        );
        if (in_array($payment, $pin_method)) {
        $form['fraud_prevention'] = array(
            '#type'             => 'fieldset',
            '#title'            => t('Risk and control management'),
            '#collapsible'      => TRUE,
            '#collapsed'        => TRUE,
            '#states'           => array('invisible' => array(':input[name="novalnet_cc_cc3d"]' => array('checked' => TRUE))));
        $form['fraud_prevention'][$payment . '_pin_method'] = array(
            '#type'                 => 'radios',
            '#title'                => t('Enable fraud prevention'),
            '#description'          => t('To authenticate the buyer for a transaction, the PIN or E-Mail will be automatically generated and sent to the buyer. This service is only available for customers from DE, AT, CH'),
            '#default_value'        => variable_get($payment . '_pin_method', '0'),
            '#options'              => array(0 => t('None'), 'callback' => t('PIN by callback'), 'sms' => t('PIN by SMS'), 'email' => t('Reply via E-mail'))
        );
        $form['fraud_prevention'][$payment . '_pin_amount'] = array(
            '#type'           => 'textfield',
            '#title'          => t('Minimum value of goods for the fraud module (in cents)'),
            '#description'    => t('Enter the minimum value of goods from which the fraud module should be activated'),
            '#default_value'  => variable_get($payment . '_pin_amount', ''),
            '#element_validate' => array('uc_novalnet_element_validate_number'),
            '#states'           => array('invisible' => array('input[name="' . $payment . '_pin_method"]' => array('value' => 0))),
        );
        }
        if (in_array($payment, array('novalnet_sepa', 'novalnet_invoice'))) {
        $comment = t('Enter the number of days after which the payment should be processed (must be greater than 6 days)');
        $title   = t('SEPA payment duration (in days)');
        if ($payment == 'novalnet_invoice') {
        $comment = t('Enter the number of days to transfer the payment amount to Novalnet (must be greater than 7 days). In case if the field is empty, 14 days will be set as due date by default');
        $title   = t('Payment due date (in days)');
        }
        $form[$payment . '_due_date'] = array(
            '#type'             => 'textfield',
            '#title'            => $title,
            '#description'      => $comment,
            '#default_value'    => variable_get($payment . '_due_date', ''),
            '#element_validate' => array('uc_novalnet_element_validate_duedate'),
        );
        }
        $form[$payment . '_user_info'] = array(
            '#type'              => 'textfield',
            '#title'             => t('Notification for the buyer'),
            '#description'       => t('The entered text will be displayed on the checkout page'),
            '#default_value'     => variable_get($payment . '_user_info', ''),
            '#element_validate'  => array('uc_novalnet_referid')
        );
        $form[$payment . '_ref1'] = array(
            '#type'              => 'textfield',
            '#title'             => t('Transaction reference 1'),
            '#default_value'     => variable_get($payment . '_ref1', ''),
            '#description'       => t('This reference will appear in your bank account statement'),
            '#element_validate'  => array('uc_novalnet_referid')
        );
        $form[$payment . '_ref2'] = array(
            '#type'              => 'textfield',
            '#title'             => t('Transaction reference 2'),
            '#default_value'     => variable_get($payment . '_ref1', ''),
            '#description'       => t('This reference will appear in your bank account statement'),
            '#element_validate'  => array('uc_novalnet_referid')
        );
        return $form;
    }

/**
* Payment method callback function: display the payment form
* @param  object $order
* @param  array $form_state
* @return array $form
*/ 

    function uc_novalnet_getform($order, $form_state) {
        $form = array();

        $payment = $order->payment_method;   $config_data = basic_params();

        if ((isset($_SESSION['novalnet_session'][$payment]['amount']) && $_SESSION['novalnet_session'][$payment]['amount'] != sprintf('%0.2f', $order->order_total) * 100 )) { 
            unset($_SESSION['novalnet_session']);  
            drupal_set_message(t('The order amount has been changed, please proceed with the new order'), 'error');       
        }
        if (in_array($payment, array('novalnet_cc', 'novalnet_sepa')) && !isset($_SESSION['novalnet_session'][$payment]['tid'])) {
            $module_url = drupal_get_path('module', 'uc_novalnet');
            $module_url = url('', array('relative' => TRUE)) . $module_url;
            $form['nn_authcode'] = array(
                '#type'           => 'hidden',
                '#default_value'  => $config_data['auth_code'],
                '#attributes' => array('id' => 'nn_authcode'),
                '#prefix'     => '<noscript><div class="messages error">' . t('Please enable the Javascript in your browser to proceed further with the payment') . '</div><input type="hidden" name="noscript"><div style="display:none"></noscript>'
            );
            $form['nn_uniqueid'] = array(
                '#type'           => 'hidden',
                '#default_value'  => randomstring(),
                '#attributes' => array('id' => 'nn_uniqueid'),
            );
            $form['nn_vendor'] = array(
                '#type'           => 'hidden',
                '#default_value'  => $config_data['vendor_id'],
                '#attributes' => array('id' => 'nn_vendor'),
            );
            $form['nn_form_loading'] = array(
                '#type'    => 'markup',
                '#markup'  => '<div id="loading-img"><img src="' . $module_url . '/img/novalnet_loader.gif"></div>'
            );
            $form_fileds  = ($payment == 'novalnet_cc') ? uc_novalnet_get_cc_form($order) : uc_novalnet_get_sepa_form($order);
            $form = array_merge($form, $form_fileds);
        }
        $pin_method_array = ($payment == 'novalnet_cc' && variable_get('novalnet_cc_cc3d')) ? array('novalnet_sepa', 'novalnet_invoice') : array('novalnet_sepa', 'novalnet_cc', 'novalnet_invoice');
        if ((in_array($payment, $pin_method_array) && variable_get($payment . '_pin_method') && ( !variable_get($payment . '_pin_amount') || $order->order_total * 100 >= variable_get($payment . '_pin_amount'))) || isset($_SESSION['novalnet_session'][$payment]['tid'])) {
            if (!isset($_SESSION['novalnet_session'][$payment]['tid'])) {
                if (variable_get($payment . '_pin_method') == 'email') {
                    $title = t('E-mail address');
                    $id = 'fraud_email';
                    $default_value = isset($order->primary_email) ? $order->primary_email : '';
                } 
                else {
                    $title = (variable_get($payment . '_pin_method') == 'sms') ? t('Mobile number') : t('Telephone number');
                    $id = 'fraud_number';
                    $default_value = isset($order->billing_phone) ? $order->billing_phone : '';
                }
                $form[$order->payment_method . '_pinkey'] = array(
                    '#type'             => 'textfield',
                    '#title'            => $title,
                    '#default_value'    => $default_value,
                    '#attributes'       => array('autocomplete' => 'off', 'id' => $id),
                    '#required'         => TRUE,
                    '#size'             => 25,
                    '#element_validate' => array('uc_novalnet_element_validate_fraud'),
                );
            }
            else {
                if (variable_get($payment . '_pin_method') != 'email') {
                $form[$order->payment_method . '_pin'] = array(
                    '#type'             => 'textfield',
                    '#title'            => t('Transaction PIN') . ' <span class="form-required" title="This field is required.">*</span>',
                    '#size'             => 20,
                    '#element_validate' => array('uc_novalnet_element_validate_pin'),
                    '#attributes'       => array('autocomplete' => 'off')
                );
                $form[$order->payment_method . '_forget_pin'] = array(
                    '#type'             => 'checkbox',
                    '#title'            => t('Forgot your PIN?'),
                );
                }
            }
        }

        $desc = uc_novalnet_payment_methods($payment);

        $test_mode_info = (variable_get($payment . '_test_mode', 0)) ? '<br /><div style =color:red;>' . t('Please Note: The payment will be processed in the test mode therefore amount for this transaction will not be charged') . '</div>' : '';
        $user_info = ((bool) variable_get($payment . '_user_info', '')) ? '<br/>' . variable_get($payment . '_user_info', '') . '<br />' : '';
        $subs_info = ''; 
        if ((bool) $config_data['enable_subscription'] && !in_array($payment, $config_data['subscription_payments']))
        $subs_info = '<div style =color:red;>' . $config_data['subscription_info'] . '</div>';
        $form[$payment . 'payment_info'] = array(
                            '#markup' => '<div style="clear:both">' . $desc['desc'] . $test_mode_info . $user_info . $subs_info . '</div>',
                            ); 
        return $form;
    }

/**
* Payment method callback function: display the payment form of Credit Card
* @param  object $order
* @return array $form
*/ 
    function uc_novalnet_get_cc_form($order) {
        $module_url = drupal_get_path('module', 'uc_novalnet');
        $module_url = url('', array('relative' => TRUE)) . $module_url;
        $cc_owner = trim($order->billing_first_name . ' ' . $order->billing_last_name);
        $card_types = array('' => t('--Select--'), 'VI' => 'Visa', 'MC' => 'MasterCard');
        if ((bool) variable_get('novalnet_cc_amex', 0))
        $card_types['AE'] ='AMEX';
        $form['nn_cc_hash'] = array(
            '#type'             => 'hidden',
            '#default_value'    => isset($_SESSION['novalnet_session']['novalnet_cc']['pan_hash']) ? $_SESSION['novalnet_session']['novalnet_cc']['pan_hash'] :  '' ,
            '#attributes'       => array('id' => 'nn_cc_hash'),
            '#element_validate' => array('uc_novalnet_nnccform_validate'),
        );
        $form['nn_cc_valid'] = array(
            '#type'           => 'hidden',
            '#default_value'  => 0,
            '#attributes' => array('id' => 'nn_cc_valid'),
        );
        $form['nncc_type'] = array(
            '#type' => 'select',
            '#title' => t('Type of card'),
            '#attributes' => array('id' => 'nncc_type'),
            '#options' => $card_types,
            '#required'      => TRUE,
            '#default_value' => '',
        );
        $form['nncc_owner'] = array(
            '#type' => 'textfield',
            '#title' => t('Card holder name'),
            '#default_value' => $cc_owner,
            '#attributes' => array('autocomplete' => 'off', 'id' => 'nncc_owner', 'onkeypress' => 'return sepa_validate_account_number(event)'),
            '#size' => 27,
            '#required'      => TRUE,
            '#maxlength' => 64,
        );
        $form['nncc_number'] = array(
            '#type'   => 'markup',
            '#markup' => '<div class="form-item form-type-textfield form-item-panes-payment-details-nncc-number">
              <label for="edit-panes-payment-details-nncc-number">' . t("Card number") . '<span class="form-required" title="This field is required."> *</span></label>
                      <input type="text" class="form-text" maxlength="35" size="27" value="" id="nncc_number" onkeypress="return ccnumber_validate(event);" autocomplete="off">
              </div>',
        );
        $config_data = basic_params();
        $form['nncc_form_refil'] = array(
            '#type'           => 'hidden',
            '#default_value'  => $config_data['refill'],
            '#attributes' => array('id' => 'nncc_form_refil'),
        );
        $year_limit = trim(variable_get('novalnet_cc_year_limit', '25')) ? trim(variable_get('novalnet_cc_year_limit', '25')) : 25  ;
        $year_limit = (date('Y')+$year_limit);
        for ($year = date('Y') ; $year < $year_limit; $year++) {
        $exp_year[$year] = $year;
        }

        $form['nncc_exp_month'] = uc_select_month(t('Expiry date'), date('m'));
        $form['nncc_exp_month']['#attributes'] = array('id' => 'nncc_exp_month');
        $form['nncc_exp_month']['#required'] = TRUE;

        $form['nncc_exp_year'] = array(
            '#type'          => 'select',
            '#attributes'    => array('id' => 'nncc_exp_year'),
            '#options'       => $exp_year,
            '#default_value' => t('--year--')
        );
        $form['nncc_cvc'] = array(
            '#type'          => 'textfield',
            '#title'         => t('CVC/CVV/CID'),
            '#default_value' => '',
            '#required'      => TRUE,
            '#attributes'    => array('autocomplete' => 'off', 'id' => 'nncc_cvc', 'onkeypress' => 'return ccnumber_validate(event);'),
            '#size'          => variable_get('novalnet_cc_amex', 1) ? 4 : 3,
            '#field_suffix'  => '<div id="cvc-hint" onmouseover="show_cchint();" onmouseout="hide_cchint();">
                                <img src="' . $module_url . '/img/cvc_hint.png">
                                <div id="cvc-hint-img"><img src="' . $module_url . '/img/creditcard_cvc.png"></div>
                              </div>
                              <span id="invalid_error_msg" style="display:none">' . t('Your credit card details are invalid') . '</span><noscript></div></noscript>'
        );
        return $form;
    }

/**
* Payment method callback function: display the payment form of Direct Debit SEPA
* @param  object $order
* @return array $form
*/ 

    function uc_novalnet_get_sepa_form($order) {
        global $user;
        $module_url = drupal_get_path('module', 'uc_novalnet');
        $module_url = url('', array('relative' => TRUE)) . $module_url;
        include_once DRUPAL_ROOT . '/includes/locale.inc';
        $billing_country          = uc_get_country_data(array('country_id' => $order->billing_country));
        $country                  = $billing_country[0]['country_iso_code_2'];
        $config_data              = basic_params();
        $sepa_refill              = ((bool) variable_get('sepa_refill', 0) && variable_get('sepa_refill' . $user->uid, '') && empty($_SESSION['novalnet_session']['novalnet_sepa']['nn_sepa_hash']) && (empty($_SESSION['nn_payment']) || (!empty($_SESSION['nn_payment']) && $_SESSION['nn_payment'] == 'novalnet_sepa'))) ? variable_get('sepa_refill' . $user->uid, '') : ''; 
        $sepa_owner               = trim($order->billing_first_name . ' ' . $order->billing_last_name);
        $form['nnsepa_owner'] = array(
            '#type'          => 'textfield',
            '#title'         => t('Account holder'),
            '#default_value' => $sepa_owner,
            '#attributes'    => array('autocomplete' => 'off', 'id' => 'nnsepa_owner', 'onkeypress' => 'return sepa_validate_account_number(event)'),
            '#size'          => 25,
            '#required'      => TRUE,
            '#maxlength'     => 64,
        );
        $countries = country_get_list();
        $form['nnsepa_country'] = array(
            '#type'       => 'select',
            '#title'      => t('Bank country'),
            '#attributes' => array('id' => 'nnsepa_country'),
            '#options'    => $countries,
            '#required'   => TRUE,
            '#maxlength'  => 20,
            '#default_value' => ($country  != '') ? $country : 'DE' ,
        );   
        $form['nnsepa_form_refil'] = array(
            '#type'           => 'hidden',
            '#default_value'  => !empty($sepa_refill) ? 1 : $config_data['refill'],
            '#attributes'     => array('id' => 'nnsepa_form_refil'),
        );
        $form['nnsepa_acdet'] = array(
            '#type'   => 'markup',
            '#markup' => '<div class="form-item form-type-textfield form-item-panes-payment-details-nnsepa-acno">
                <label for="edit-panes-payment-details-nnsepa-acno">' . t('IBAN or Account number') . ' <span class="form-required" title="This field is required.">*</span></label>
                <input type="text" class="form-text" maxlength="25" size="25" value="" id="nnsepa_acno" onkeypress="return sepa_validate_account_number(event)" autocomplete="off">
              </div>
              <div class="form-item form-type-textfield form-item-panes-payment-details-nnsepa-iban" id="nnsepa_iban_div">
                <label>' . t('IBAN') . ' : </label><label id="frm_iban"></label>
              </div>
              <div class="form-item form-type-textfield form-item-panes-payment-details-nnsepa-bcode" >
                <label for="edit-panes-payment-details-nnsepa-bcode">' . t('BIC or Bank code') . ' <span class="form-required" title="This field is required.">*</span></label>
                <input type="text" class="form-text" maxlength="25" size="25" value="" id="nnsepa_bcode" onkeypress="return sepa_validate_account_number(event)" autocomplete="off">
              </div>
              <div class="form-item form-type-textfield form-item-panes-payment-details-nnsepa-bic" id="nnsepa_bic_div">
                <label>' . t('BIC') . ' : </label><label id="frm_bic" ></label>
              </div>
              <span id="sepa_error_msg" style="display:none">' . t('Your account details are invalid') . '</span>
              <span id="sepa_confirm_error_msg" style="display:none">' . t('Please accept the SEPA direct debit mandate') . '</span>'
        );
        $form['nnsepa_ibanconf'] = array(
            '#type'          => 'checkbox',
            '#default_value' => '',
            '#attributes'    => array('autocomplete' => 'off', 'id' => 'nnsepa_ibanconf', 'onclick' => 'return nnsepa_confirm();'),
            '#size'          => 20,
            '#maxlength'     => 64,
            '#description'   => t("I hereby grant the SEPA direct debit mandate and confirm that the given IBAN and BIC are correct"),
            '#element_validate' => array('uc_novalnet_nnsepaform_validate'),
        );
        $form['nn_sepa_hash'] = array(
            '#type'           => 'hidden',
            '#default_value'  => !empty($sepa_refill) ? $sepa_refill : (!empty($_SESSION['novalnet_session']['novalnet_sepa']['nn_sepa_hash']) ? $_SESSION['novalnet_session']['novalnet_sepa']['nn_sepa_hash'] : ""),
            '#attributes'     => array('id' => 'nn_sepa_hash'),
        );
        $form['nnsepa_holder_name'] = array(
            '#type'           => 'hidden',
            '#default_value'  => isset($_SESSION['novalnet_session']['novalnet_sepa']['nnsepa_owner']) ? $_SESSION['novalnet_session']['novalnet_sepa']['nnsepa_owner'] : $sepa_owner,
            '#attributes'     => array('id' => 'nnsepa_holder_name'),
        );
        $form['nn_sepa_hash_generated'] = array(
            '#type'           => 'hidden',
            '#default_value'  => 0,
            '#attributes'     => array('id' => 'nn_sepa_hash_generated'),
        );    
        $form['sepa_from_acno'] = array(
            '#type'           => 'hidden',
            '#default_value'  => isset($_SESSION['novalnet_session']['novalnet_sepa']['sepa_from_acno']) ? $_SESSION['novalnet_session']['novalnet_sepa']['sepa_from_acno'] : 0,
            '#attributes'     => array('id' => 'sepa_from_acno'),
        );
        $form['nnsepa_overay'] = array(
            '#type'          => 'markup',
            '#markup'        => '<div id="overlay-dark"></div>
            <div id="account_details" class="account_details">
              <table>
                <tr>
                  <td colspan="2" class="header">' . t('Direct debit SEPA mandate confirmation') . '</td>
                </tr>
                <tr>
                  <td width="200px">' . t('Creditor') . '</td><td id="nnsepa_creator"></td>
                </tr>
                <tr>
                  <td>' . t('Creditor identifier') . '</td><td id="nnsepa_iden"></td>
                </tr>
                <tr>
                  <td>' . t('Mandate reference') . '</td><td id="nnsepa_mandate_ref"></td>
                </tr>
                <tr>
                  <td colspan="2">' . t('By granting this mandate form, I authorize (A) the creditor to send instructions to my bank to debit my account and (B) my bank to debit my account in accordance with the instructions from the creditor for this and future payments.') . '
                  <br/><br/>' . t('As part of my rights, I am entitled to a refund from my bank under the terms and conditions of my agreement with my bank. A refund must be claimed within eight weeks from the date on which my account was debited.') . '</td>
                </tr>
                <tr>
                  <td>' . t("Name of the payee") . '</td><td class="nnsepa_holder"></td>
                </tr>
                <tr id="ovr_company">
                  <td>' . t('Company name') . '</td><td id="nnsepa_company"></td>
                </tr>
                <tr>
                  <td>' . t('Street name and number') . '</td><td id="nnsepa_address"></td>
                </tr>
                <tr>
                  <td>' . t('Postal code and City') . '</td><td class="nnsepa_city"></td>
                </tr>
                <tr>
                  <td>' . t('Country') . '</td><td id="nnsepa_country_o"></td>
                </tr>
                <tr>
                  <td>' . t('E-Mail') . '</td><td nnsepa_email>' . $order->primary_email . '</td>
                </tr>
                <tr>
                  <td>' . t('IBAN') . '</td><td id="over_iban"></td>
                </tr>
                <tr id="ovr_bic_tr">
                  <td>' . t('BIC') . '</td><td id="over_bic"></td>
                </tr>
                <tr>
                  <td colspan="2"><span class="nnind_city"></span>, <span>' . date("d.m.Y") . '</span>, <span class="nnsepa_holder"></span></td>
                </tr> </table>
                <div>
                    <input id ="man_confirm" type="button" value="' . t('Confirm') . '" onclick="return sepa_mandate_confirm();">
                    <input id ="man_close" type="button" value="' . t('Cancel') . '" onclick="return sepa_mandate_close();">
                    <img src="' . $module_url . '/img/novalnet_logo.png">
                </div>
            </div>',
        );
        return $form;
    }

/**
* Payment method callback function: additional params formation for redirect payments 
* @param  object $order
* @param  array $form
* @param  array $form_state
* @return array $form
*/ 
    function uc_novalnet_redirecrt_form($form, &$form_state, $order) {
        $data = array();
        uc_novalnet_getparams( $data, $order);
        $return_url                   = url('novalnet_order/review',  array('absolute' => TRUE));
        $data['return_url']           = $return_url;
        $data['return_method']        = 'POST';
        $data['error_return_url']     = $return_url;
        $data['error_return_method']  = 'POST';
        foreach ($data as $name => $value) {
            $form[$name] = array('#type' => 'hidden', '#value' => $value);
        }
        uc_novalnet_save_log($order->order_id, $data, 'request');  
        $form['#action'] = uc_novalnet_get_url($order->payment_method );
        $form['actions'] = array('#type' => 'actions');
        $form['actions']['submit'] = array(
            '#type' => 'submit',
            '#value' => t('Submit order'),
        );
        return $form;
    }

/**
* Payment method callback function: to store transaction request and responses
* @param  int $order_id
* @return array $form
*/
    function uc_novalnet_payment_logs($order_id) {
        $callback = array();
        $order    = db_query("SELECT logtype, logvalue, logtime FROM  {uc_novalnet_logs} WHERE order_id = :id", array(':id' => $order_id->order_id)) -> fetchAll();

        foreach ($order as $val) {
          switch ($val->logtype) {
            case 'request':
                $request[] = $val;
                break;
            case 'response':
                $response[] = $val;
                break;
            case 'callback':
                $callback[] = $val;
                break;
          }
        }

        $form['novalnet_logs'] = array(
            '#type' => 'fieldset',
            '#title' => t('Novalnet Transactions Log') . '  -  ' . $order_id->order_id,
            '#collapsible'  => TRUE,
            '#collapsed'    => FALSE
        );
        $form['novalnet_logs']['request'] = array(
            '#type' => 'fieldset',
            '#title' => t('Novalnet Request Data'),
            '#collapsible'  => TRUE,
            '#collapsed'     => TRUE
        );
        $form['novalnet_logs']['response'] = array(
            '#type' => 'fieldset',
            '#title' => t('Novalnet Response Data'),
            '#collapsible'  => TRUE,
            '#collapsed'     => TRUE
        );
        $form['novalnet_logs']['callback'] = array(
            '#type' => 'fieldset',
            '#title' => t('Novalnet Merchant script'),
            '#collapsible'  => TRUE,
            '#collapsed'     => TRUE
        );
        $message  = t('No Results Found');
        if (!empty($request)) {
            foreach ($request as $rqval) {
                $form['novalnet_logs']['request']['request_data'][] = array(
                    '#type' => 'textarea',
                    '#title' => date('d.m.Y H:i:s ', $rqval->logtime),
                    '#value' => print_r(unserialize($rqval->logvalue), TRUE),
                    '#attributes' => array('readonly' => 'readonly')
                );
            }
        }
        else $form['novalnet_logs']['request']['request_data']['#markup'] = $message;
        if (!empty($response)) {
            foreach ($response as $rsval) {
                $form['novalnet_logs']['response']['response_data'][] = array(
                    '#type' => 'textarea',
                    '#title' => date('d.m.Y H:i:s ', $rsval->logtime),
                    '#value' => print_r(unserialize($rsval->logvalue), TRUE),
                    '#attributes' => array('readonly' => 'readonly')
                );
            }
        }
        else $form['novalnet_logs']['response']['request_data']['#markup'] = $message;
        if (!empty($callback)) {
            foreach ($callback as $urls) {
                $form['novalnet_logs']['callback']['response_data'][] = array(
                    '#type' => 'textarea',
                    '#title' => date('d.m.Y H:i:s ', $urls->logtime),
                    '#value' => print_r(unserialize($urls->logvalue), TRUE),
                    '#attributes' => array('readonly' => 'readonly')
                );
            }
        }
        else $form['novalnet_logs']['callback']['request_data']['#markup'] =  $message;
        return $form;
    }
